from leapp.libraries.common import rhsm
from leapp.libraries.common.config import get_env
from leapp.libraries.stdlib import api, CalledProcessError, run


def _insights_register():
    try:
        run(['insights-client', '--register'])
        api.current_logger().info('Automatically registered into Red Hat Insights')
    except (CalledProcessError) as err:
        # TODO(mmatuska) produce post-upgrade report?
        api.current_logger().error(
            'Automatic registration into Red Hat Insights failed: {}'.format(err)
        )


def process():
    if rhsm.skip_rhsm() or get_env('LEAPP_NO_INSIGHTS_REGISTER', '0') == '1':
        api.current_logger().debug(
            'Skipping registration into Insights due to --no-insights-register'
            ' or LEAPP_NO_INSIGHTS_REGISTER=1 set'
        )
        return

    _insights_register()



from leapp.actors import Actor
from leapp.libraries.actor import vsftpdconfigcheck
from leapp.models import TcpWrappersFacts, VsftpdFacts
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class VsftpdConfigCheck(Actor):
    """
    Checks whether the vsftpd configuration is supported in RHEL-8. Namely checks that
    configuration files don't set tcp_wrappers=YES and vsftpd-related configuration is
    not present in tcp_wrappers configuration files at the same time.
    """

    name = 'vsftpd_config_check'
    consumes = (TcpWrappersFacts, VsftpdFacts,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag,)

    def process(self):
        try:
            vsftpd_facts = next(self.consume(VsftpdFacts))
        except StopIteration:
            return
        tcp_wrappers_facts = next(self.consume(TcpWrappersFacts))
        vsftpdconfigcheck.check_config_supported(tcp_wrappers_facts, vsftpd_facts)



from leapp.actors import Actor
from leapp.libraries.actor import checkinitramfstasks
from leapp.models import Report, TargetInitramfsTasks, UpgradeInitramfsTasks
from leapp.tags import IPUWorkflowTag, TargetTransactionChecksPhaseTag


class CheckInitramfsTasks(Actor):
    """
    Inhibit the upgrade if conflicting "initramfs" tasks are detected

    It's possible that some actors could provide conflicting requests
    (e.g. install system dracut module A & custom (from own path) dracut
    module A). This actor prevents system from upgrading if conflicting tasks
    for the upgrade or target initramfses are detected.
    """

    name = 'check_initramfs_tasks'
    consumes = (UpgradeInitramfsTasks, TargetInitramfsTasks)
    produces = (Report,)
    tags = (TargetTransactionChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        checkinitramfstasks.process()



from leapp import reporting
from leapp.libraries.common.tcpwrappersutils import config_applies_to_daemon
from leapp.reporting import create_report


def check_config_supported(tcpwrap_facts, vsftpd_facts):
    bad_configs = [config.path for config in vsftpd_facts.configs if config.tcp_wrappers]
    if bad_configs and config_applies_to_daemon(tcpwrap_facts, 'vsftpd'):
        list_separator_fmt = '\n    - '
        create_report([
            reporting.Title('Unsupported vsftpd configuration'),
            reporting.Summary(
                    'tcp_wrappers support has been removed in RHEL-8. '
                    'Some configuration files set the tcp_wrappers option to true and '
                    'there is some vsftpd-related configuration in /etc/hosts.deny '
                    'or /etc/hosts.allow. Please migrate it manually. '
                    'The list of problematic configuration files:{}{}'.
                    format(
                        list_separator_fmt,
                        list_separator_fmt.join(bad_configs)
                    )
            ),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.SERVICES, reporting.Groups.NETWORK]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.ExternalLink(
                title='Replacing TCP Wrappers in RHEL 8',
                url='https://access.redhat.com/solutions/3906701'
            ),
            reporting.RelatedResource('package', 'tcp_wrappers'),
            reporting.RelatedResource('package', 'vsftpd'),
        ] + [reporting.RelatedResource('file', str(bc)) for bc in bad_configs])



from leapp import reporting
from leapp.libraries.stdlib import api
from leapp.models import BlackListCA, BlackListError


# just like replace except it starts from the back
# of the string
def rreplace(s, old, new, count):
    return s[::-1].replace(old[::-1], new[::-1], count)[::-1]


def process():
    moving = {}
    commaTarget = {}
    deleting = []
    # process all the BlackListCA events into a single report
    # first collect all the files moving to the same target.
    # as well as any source directories that will be deleted
    for ca in api.consume(BlackListCA):
        if ca.targetDir not in commaTarget:
            commaTarget[ca.targetDir] = ''
        if ca.targetDir not in moving:
            moving[ca.targetDir] = ''
        moving[ca.targetDir] = moving[ca.targetDir] + commaTarget[ca.targetDir] + ca.source
        commaTarget[ca.targetDir] = ', '
        if ca.sourceDir not in deleting:
            deleting.append(ca.sourceDir)

    # now make our lists of files and targets into a single string
    comma = ''
    reportString = ''
    for key in moving:
        # replace the last ', ' with ' and '
        moveString = rreplace(moving[key], ', ', ' and ', 1)
        reportString = reportString + comma + "{} will be moved to {}".format(moveString, key)
        comma = ': '
    reportString = rreplace(reportString, ': ', ' and ', 1).replace(': ', ', ')

    # finally make a string our of the removed directories
    comma = ''
    deleteString = ''
    for d in deleting:
        deleteString = deleteString + comma + d
        comma = ', '
    deleteString = rreplace(deleteString, ', ', ' and ', 1)

    # finally make a string of the
    if moving:
        reporting.create_report([
            reporting.Title('Distrusted CA certificates will be moved from blacklist to blocklist'),
            reporting.Summary(
                'The directories which store user and administrator supplied '
                'distrusted certificates have change names from blacklist in '
                'RHEL8 to blocklist in RHEL9. As a result {} and '
                '{} will be deleted.'.format(reportString, deleteString)),
            reporting.Severity(reporting.Severity.INFO),
            reporting.Groups([reporting.Groups.SECURITY]),
            reporting.Groups([reporting.Groups.AUTHENTICATION])
        ])
    for ble in api.consume(BlackListError):
        reporting.create_report([
            reporting.Title('Could not access blacklist directory'),
            reporting.Summary(
                'The directories which stores user and administrator supplied '
                'distrusted certificates has change names from blacklist in '
                'RHEL8 to blocklist in RHEL9. But we are unable to access the '
                'RHEL8 directory {} because {}. You can clear this error by '
                'correcting the condition, or by moving the contents to {} '
                'and removing {} completely'
                '. '.format(ble.sourceDir, ble.error, ble.targetDir, ble.sourceDir)),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.SECURITY]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Groups([reporting.Groups.AUTHENTICATION])
        ])




"""
This is not regular library.

The library is supposed to be used only for testing purposes. Import of the
library is expected only inside test files.
"""

from leapp.models import EnvVar, IPUConfig, OSRelease, Version

CONFIG = IPUConfig(
    leapp_env_vars=[EnvVar(name='LEAPP_DEVEL', value='0')],
    os_release=OSRelease(
        release_id='rhel',
        name='Red Hat Enterprise Linux Server',
        pretty_name='RHEL',
        version='7.6 (Maipo)',
        version_id='7.6'
    ),
    version=Version(
        source='7.6',
        target='8.0'
    ),
    architecture='x86_64',
    kernel='3.10.0-957.43.1.el7.x86_64',
)

CONFIG_NO_NETWORK_RENAMING = IPUConfig(
    leapp_env_vars=[EnvVar(name='LEAPP_DEVEL', value='0'), EnvVar(name='LEAPP_NO_NETWORK_RENAMING', value='1')],
    os_release=OSRelease(
        release_id='rhel',
        name='Red Hat Enterprise Linux Server',
        pretty_name='RHEL',
        version='7.6 (Maipo)',
        version_id='7.6'
    ),
    version=Version(
        source='7.6',
        target='8.0'
    ),
    architecture='x86_64',
    kernel='3.10.0-957.43.1.el7.x86_64',
)

CONFIG_ALL_SIGNED = IPUConfig(
    leapp_env_vars=[EnvVar(name='LEAPP_DEVEL_RPMS_ALL_SIGNED', value='1')],
    os_release=OSRelease(
        release_id='rhel',
        name='Red Hat Enterprise Linux Server',
        pretty_name='RHEL',
        version='7.6 (Maipo)',
        version_id='7.6'
    ),
    version=Version(
        source='7.6',
        target='8.0'
    ),
    architecture='x86_64',
    kernel='3.10.0-957.43.1.el7.x86_64',
)

CONFIG_S390X = IPUConfig(
    os_release=OSRelease(
        release_id='rhel',
        name='Red Hat Enterprise Linux Server',
        pretty_name='RHEL',
        version='7.6 (Maipo)',
        version_id='7.6'
    ),
    version=Version(
        source='7.6',
        target='8.0'
    ),
    architecture='s390x',
    kernel='3.10.0-957.43.1.el7.x86_64',
)



from leapp import reporting
from leapp.libraries.common import grub
from leapp.libraries.common.config import architecture
from leapp.libraries.stdlib import api, CalledProcessError, config, run
from leapp.models import FirmwareFacts


def update_grub_core(grub_devs):
    """
    Update GRUB core after upgrade from RHEL7 to RHEL8

    On legacy systems, GRUB core does not get automatically updated when GRUB packages
    are updated.
    """

    successful = []
    failed = []
    for dev in grub_devs:
        cmd = ['grub2-install', dev]
        if config.is_debug():
            cmd += ['-v']
        try:
            run(cmd)
        except CalledProcessError as err:
            api.current_logger().warning('GRUB core update on {} failed: {}'.format(dev, err))
            failed.append(dev)
            continue

        successful.append(dev)

    if failed:
        if successful:
            # partial failure
            summary = (
                'GRUB was successfully updated on the following devices: {},\n'
                'however GRUB update failed on the following devices: {}'
            ).format(', '.join(successful), ', '.join(failed))
        else:
            summary = 'Leapp failed to update GRUB on {}'.format(', '.join(failed))

        reporting.create_report([
            reporting.Title('GRUB core update failed'),
            reporting.Summary(summary),
            reporting.Groups([reporting.Groups.BOOT]),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Remediation(
                hint='Please run "grub2-install <GRUB_DEVICE>" manually after upgrade'
            )
        ])
    else:
        reporting.create_report([
            reporting.Title('GRUB core successfully updated'),
            reporting.Summary('GRUB core on {} was successfully updated'.format(', '.join(successful))),
            reporting.Groups([reporting.Groups.BOOT]),
            reporting.Severity(reporting.Severity.INFO)
        ])


def process():
    if architecture.matches_architecture(architecture.ARCH_S390X):
        return
    ff = next(api.consume(FirmwareFacts), None)
    if ff and ff.firmware == 'bios':
        grub_devs = grub.get_grub_devices()
        if grub_devs:
            update_grub_core(grub_devs)
        else:
            api.current_logger().warning('Leapp could not detect GRUB devices')



import pytest

from leapp.exceptions import StopActorExecutionError
from leapp.libraries import stdlib
from leapp.libraries.actor import scankernel
from leapp.libraries.common import kernel as kernel_lib
from leapp.libraries.common.testutils import CurrentActorMocked, logger_mocked
from leapp.libraries.stdlib import api
from leapp.models import InstalledTargetKernelInfo, InstalledTargetKernelVersion, KernelInfo, RPM
from leapp.utils.deprecation import suppress_deprecation

TARGET_KERNEL_NEVRA = 'kernel-core-1.2.3-4.el9.x86_64'
TARGET_RT_KERNEL_NEVRA = 'kernel-rt-core-1.2.3-4.rt56.7.el9.x86_64'
OLD_KERNEL_NEVRA = 'kernel-core-0.1.2-3.el8.x86_64'
OLD_RT_KERNEL_NEVRA = 'kernel-rt-core-0.1.2-3.rt4.5.el8.x86_64'


class MockedRun(object):

    def __init__(self, stdouts):
        # stdouts should be dict of list of strings: { str: [str1,str2,...]}
        self._stdouts = stdouts

    def __call__(self, *args, **kwargs):
        for key in ('kernel-core', 'kernel-rt-core'):
            if key in args[0]:
                return {'stdout': self._stdouts.get(key, [])}
        return {'stdout': []}


@suppress_deprecation(InstalledTargetKernelVersion)
def assert_produced_messages_are_correct(produced_messages, expected_target_nevra, initramfs_path, kernel_img_path):
    target_evra = expected_target_nevra.replace('kernel-core-', '').replace('kernel-rt-core-', '')
    installed_kernel_ver = [msg for msg in produced_messages if isinstance(msg, InstalledTargetKernelVersion)]
    assert len(installed_kernel_ver) == 1, 'Actor should produce InstalledTargetKernelVersion (backwards compat.)'
    assert installed_kernel_ver[0].version == target_evra

    installed_kernel_info = [msg for msg in produced_messages if isinstance(msg, InstalledTargetKernelInfo)]
    assert len(installed_kernel_info) == 1
    assert installed_kernel_info[0].pkg_nevra == expected_target_nevra

    assert installed_kernel_info[0].initramfs_path == initramfs_path
    assert installed_kernel_info[0].kernel_img_path == kernel_img_path


@pytest.mark.parametrize(
    ('is_rt', 'expected_target_nevra', 'stdouts'),
    [
        (False, TARGET_KERNEL_NEVRA, {'kernel-core': [OLD_KERNEL_NEVRA, TARGET_KERNEL_NEVRA]}),
        (False, TARGET_KERNEL_NEVRA, {'kernel-core': [TARGET_KERNEL_NEVRA, OLD_KERNEL_NEVRA]}),
        (False, TARGET_KERNEL_NEVRA, {
            'kernel-core': [TARGET_KERNEL_NEVRA, OLD_KERNEL_NEVRA],
            'kernel-rt-core': [TARGET_RT_KERNEL_NEVRA, OLD_RT_KERNEL_NEVRA],
        }),
        (True, TARGET_RT_KERNEL_NEVRA, {
            'kernel-rt-core': [OLD_RT_KERNEL_NEVRA, TARGET_RT_KERNEL_NEVRA]
        }),
        (True, TARGET_RT_KERNEL_NEVRA, {
            'kernel-rt-core': [TARGET_RT_KERNEL_NEVRA, OLD_RT_KERNEL_NEVRA]
        }),
        (True, TARGET_RT_KERNEL_NEVRA, {
            'kernel-core': [TARGET_KERNEL_NEVRA, OLD_KERNEL_NEVRA],
            'kernel-rt-core': [TARGET_RT_KERNEL_NEVRA, OLD_RT_KERNEL_NEVRA],
        }),
    ]
)
def test_scaninstalledkernel(monkeypatch, is_rt, expected_target_nevra, stdouts):
    src_kernel_pkg = RPM(name='kernel-core', arch='x86_64', version='0.1.2', release='3',
                         epoch='0', packager='', pgpsig='SOME_OTHER_SIG_X')
    src_kernel_type = kernel_lib.KernelType.REALTIME if is_rt else kernel_lib.KernelType.ORDINARY
    src_kernel_info = KernelInfo(pkg=src_kernel_pkg, type=src_kernel_type, uname_r='X')

    def patched_get_boot_files(nevra):
        assert nevra == expected_target_nevra
        return scankernel.KernelBootFiles(vmlinuz_path='/boot/vmlinuz-X', initramfs_path='/boot/initramfs-X')

    result = []
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(dst_ver='9.0', msgs=[src_kernel_info]))
    monkeypatch.setattr(api, 'produce', result.append)
    monkeypatch.setattr(scankernel, 'run', MockedRun(stdouts))
    monkeypatch.setattr(scankernel, 'get_boot_files_provided_by_kernel_pkg', patched_get_boot_files)
    monkeypatch.setattr(kernel_lib, 'get_uname_r_provided_by_kernel_pkg', lambda nevra: 'uname-r')

    scankernel.process()

    assert_produced_messages_are_correct(result, expected_target_nevra, '/boot/initramfs-X', '/boot/vmlinuz-X')


@pytest.mark.parametrize(
    ('vmlinuz_path', 'initramfs_path', 'extra_kernel_rpm_files'),
    (
        ('/boot/vmlinuz-x', '/boot/initramfs-x', []),
        ('/boot/vmlinuz-x', '/boot/initramfs-x', ['/lib/modules/6.4.10-100.fc37.x86_64/vmlinuz']),
        (None, '/boot/initramfs-x', ['/lib/modules/6.4.10-100.fc37.x86_64/vmlinuz']),
        ('/boot/vmlinuz-x', None, ['/lib/modules/6.4.10-100.fc37.x86_64/vmlinuz']),
    )
)
def test_get_boot_files_provided_by_kernel_pkg(monkeypatch, vmlinuz_path, initramfs_path, extra_kernel_rpm_files):
    def mocked_run(cmd, *args, **kwargs):
        assert cmd == ['rpm', '-q', '-l', TARGET_KERNEL_NEVRA]

        output = list(extra_kernel_rpm_files)
        if vmlinuz_path:
            output.append(vmlinuz_path)
        if initramfs_path:
            output.append(initramfs_path)

        return {
            'stdout': output
        }

    monkeypatch.setattr(scankernel, 'run', mocked_run)

    if not vmlinuz_path or not initramfs_path:
        with pytest.raises(StopActorExecutionError):
            scankernel.get_boot_files_provided_by_kernel_pkg(TARGET_KERNEL_NEVRA)
    else:
        result = scankernel.get_boot_files_provided_by_kernel_pkg(TARGET_KERNEL_NEVRA)
        assert result.vmlinuz_path == vmlinuz_path
        assert result.initramfs_path == initramfs_path


def test_scaninstalledkernel_missing_rt(monkeypatch):
    src_kernel_pkg = RPM(name='kernel-rt-core', arch='x86_64', version='0.1.2', release='3',
                         epoch='0', packager='', pgpsig='SOME_OTHER_SIG_X')
    src_kernel_type = kernel_lib.KernelType.REALTIME
    src_kernel_info = KernelInfo(pkg=src_kernel_pkg, type=src_kernel_type, uname_r='X')

    result = []
    stdouts = {'kernel-core': [TARGET_KERNEL_NEVRA], 'kernel-rt-core': [OLD_RT_KERNEL_NEVRA]}

    def patched_get_boot_content(target_nevra):
        return scankernel.KernelBootFiles(vmlinuz_path='/boot/vmlinuz-X', initramfs_path='/boot/initramfs-X')

    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(dst_ver='9.0', msgs=[src_kernel_info]))
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(api, 'produce', result.append)
    monkeypatch.setattr(scankernel, 'run', MockedRun(stdouts))
    monkeypatch.setattr(scankernel, 'get_boot_files_provided_by_kernel_pkg', patched_get_boot_content)
    monkeypatch.setattr(kernel_lib, 'get_uname_r_provided_by_kernel_pkg', lambda nevra: 'uname-r')

    scankernel.process()

    assert api.current_logger.warnmsg

    assert_produced_messages_are_correct(result, TARGET_KERNEL_NEVRA, '/boot/initramfs-X', '/boot/vmlinuz-X')


def test_scaninstalledkernel_missing(monkeypatch):
    src_kernel_pkg = RPM(name='kernel-rt-core', arch='x86_64', version='0.1.2', release='3',
                         epoch='0', packager='', pgpsig='SOME_OTHER_SIG_X')
    src_kernel_type = kernel_lib.KernelType.REALTIME
    src_kernel_info = KernelInfo(pkg=src_kernel_pkg, type=src_kernel_type, uname_r='X')

    result = []

    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[src_kernel_info]))
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(api, 'produce', result.append)
    monkeypatch.setattr(scankernel, 'run', MockedRun({}))
    monkeypatch.setattr(kernel_lib, 'get_uname_r_provided_by_kernel_pkg', lambda nevra: 'uname-r')

    scankernel.process()

    assert not result



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class SapHanaManifestEntry(Model):
    """ SAP HANA manifest file key/value pair entry """
    topic = SystemInfoTopic
    key = fields.String()
    """ Key of the key value pairs in a SAP HANA manifest file """
    value = fields.String()
    """ Value of the key value pairs in a SAP HANA manifest file """


class SapHanaInstanceInfo(Model):
    """ Details of SAP HANA instances """
    topic = SystemInfoTopic

    name = fields.String()
    """ Name of the instance """
    path = fields.String()
    """ Directory of the SAP HANA instance """
    instance_number = fields.String()
    """ SAP HANA Instance number """
    running = fields.Boolean()
    """ Is the instance currently running? """
    admin = fields.String()
    """ Name of the instance administrator """
    manifest = fields.List(fields.Model(SapHanaManifestEntry))
    """ Content of the SAP HANA manifest file """


class SapHanaInfo(Model):
    """
    This message contains information collected around SAP HANA

    If SAP HANA has been detected on the machine, it is running and which versions have been detected.
    """
    topic = SystemInfoTopic
    installed = fields.Boolean(default=False)
    """ True if SAP HANA has been detected on the system """
    running = fields.Boolean(default=False)
    """ True if an instance of SAP HANA is running """
    instances = fields.List(fields.Model(SapHanaInstanceInfo))
    """ List of instance details of SAP HANA detected on the system """



from leapp.actors import Actor
from leapp.libraries.actor import checkifcfg_ifcfg as ifcfg
from leapp.models import IfCfg, InstalledRPM, Report, RpmTransactionTasks
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckIfcfg(Actor):
    """
    Ensures that ifcfg files are compatible with NetworkManager

    Checks whether the ifcfg files would work with NetworkManager's ifcfg-rh
    compatibility module -- they are of known type, well-formed and not
    explicitly disabled with NM_CONTROLLED=no.

    Makes sure relevant NetworkManager modules end up getting installed.
    """

    name = "check_ifcfg"
    consumes = (IfCfg, InstalledRPM,)
    produces = (Report, RpmTransactionTasks,)
    tags = (ChecksPhaseTag, IPUWorkflowTag,)

    def process(self):
        ifcfg.process()



from leapp.actors import Actor
from leapp.libraries.actor import setuptargetrepos
from leapp.models import (
    CustomTargetRepository,
    InstalledRPM,
    RepositoriesBlacklisted,
    RepositoriesFacts,
    RepositoriesMapping,
    RepositoriesSetupTasks,
    RHUIInfo,
    SkippedRepositories,
    TargetRepositories,
    UsedRepositories
)
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class SetupTargetRepos(Actor):
    """
    Produces list of repositories that should be available to be used by Upgrade process.

    Based on current set of Red Hat Enterprise Linux repositories, produces the list of target
    repositories. Additionally process request to use custom repositories during the upgrade
    transaction.
    """

    name = 'setuptargetrepos'
    consumes = (CustomTargetRepository,
                InstalledRPM,
                RepositoriesSetupTasks,
                RepositoriesMapping,
                RepositoriesFacts,
                RepositoriesBlacklisted,
                RHUIInfo,
                UsedRepositories)
    produces = (TargetRepositories, SkippedRepositories)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        setuptargetrepos.process()



import errno

from leapp.libraries.common.testutils import make_IOError, make_OSError


def test_make_IOError():
    exception = make_IOError(errno.ENOENT)
    assert isinstance(exception, IOError)
    assert exception.errno == errno.ENOENT

    exception = make_IOError(errno.ENOTDIR)
    assert isinstance(exception, IOError)
    assert exception.errno == errno.ENOTDIR


def test_make_OSError():
    exception = make_OSError(errno.ENOENT)
    assert isinstance(exception, OSError)
    assert exception.errno == errno.ENOENT

    exception = make_OSError(errno.ENOTDIR)
    assert isinstance(exception, OSError)
    assert exception.errno == errno.ENOTDIR



from leapp.actors import Actor
from leapp.libraries.actor import transactionworkarounds
from leapp.models import RpmTransactionTasks
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class TransactionWorkarounds(Actor):
    """
    Provides additional RPM transaction tasks based on bundled RPM packages.

    After collecting bundled RPM packages, a message with relevant data will be produced.
    """

    name = 'transaction_workarounds'
    consumes = ()
    produces = (RpmTransactionTasks,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        transactionworkarounds.process()



import os

import six

from leapp.models import InvalidRootSubdirectory, RootDirectory, RootSubdirectory


def scan_dir(root_dir=b'/'):
    """
    Scan root directory and return a RootDirectory(subdirs, invalid_subdirs) model object
    """
    subdirs = []
    invalid_subdirs = []

    def _create_a_subdir(subdir_cls, name, path):
        if os.path.islink(path):
            return subdir_cls(name=name, target=os.readlink(path))
        return subdir_cls(name=name)

    for subdir in os.listdir(root_dir):
        # Note(ivasilev) in py3 env non-utf encoded string will appear as byte strings
        # However in py2 env subdir will be always of str type, so verification if this is a valid utf-8 string
        # should be done differently than formerly suggested plain six.binary_type check
        decoded = True
        if isinstance(subdir, six.binary_type):
            try:
                subdir = subdir.decode('utf-8')
            except (AttributeError, UnicodeDecodeError):
                decoded = False
        if not decoded:
            invalid_subdirs.append(_create_a_subdir(InvalidRootSubdirectory, subdir, os.path.join(b'/', subdir)))
        else:
            subdirs.append(_create_a_subdir(RootSubdirectory, subdir, os.path.join('/', subdir)))
    return RootDirectory(items=subdirs, invalid_items=invalid_subdirs)



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class SSSDConfig8to9(Model):
    """
    SSSD configuration that is related to the upgrade process.
    """
    topic = SystemInfoTopic

    enable_files_domain_set = fields.Boolean()
    """
    True if [sssd]/enable_files_domain is explicitly set.
    """

    explicit_files_domain = fields.Boolean()
    """
    True if a domain with id_provider=files exist.
    """

    pam_cert_auth = fields.Boolean()
    """
    True if [pam]/pam_cert_auth is set to True.
    """



import xml.etree.ElementTree as ElementTree

from leapp.libraries.actor import firewalldupdatelockdownwhitelist


def test_firewalldupdatelockdownwhitelist_library():
    root = ElementTree.fromstring(
        '''<?xml version="1.0" encoding="utf-8"?>
           <whitelist>
             <command name="/usr/bin/python -Es /usr/bin/firewall-config"/>
             <command name="/usr/bin/foobar"/>
             <selinux context="system_u:system_r:NetworkManager_t:s0"/>
             <selinux context="system_u:system_r:virtd_t:s0-s0:c0.c1023"/>
             <user id="0"/>
           </whitelist>
        ''')

    assert firewalldupdatelockdownwhitelist.updateFirewallConfigCommand(
        root,
        '/usr/bin/python -Es /usr/bin/firewall-config'
    )


def test_firewalldupdatelockdownwhitelist_library_negative():
    root = ElementTree.fromstring(
        '''<?xml version="1.0" encoding="utf-8"?>
           <whitelist>
             <command name="/usr/bin/foobar"/>
           </whitelist>
        ''')

    assert not firewalldupdatelockdownwhitelist.updateFirewallConfigCommand(root, '')
    assert not firewalldupdatelockdownwhitelist.updateFirewallConfigCommand(
        root,
        '/usr/bin/python -Es /usr/bin/firewall-config'
    )

    root = ElementTree.fromstring(
        '''<?xml version="1.0" encoding="utf-8"?>
           <whitelist>
             <command name="/usr/libexec/platform-python -s /usr/bin/firewall-config"/>
             <selinux context="system_u:system_r:NetworkManager_t:s0"/>
             <selinux context="system_u:system_r:virtd_t:s0-s0:c0.c1023"/>
             <user id="0"/>
           </whitelist>
        ''')

    assert not firewalldupdatelockdownwhitelist.updateFirewallConfigCommand(
        root,
        '/usr/libexec/platform-python -s /usr/bin/firewall-config'
    )



import pytest

from leapp.libraries.actor import pluginscanner

CMD_YUM_OUTPUT = '''Loaded plugins: langpacks, my plugin, subscription-manager, product-id
Usage: yum [options] COMMAND
'''
CMD_YUM_OUTPUT_MULTILINE_BREAK_ON_HYPHEN = '''Loaded plugins: langpacks, my plugin, subscription-
              : manager, product-id
Usage: yum [options] COMMAND
'''
CMD_YUM_OUTPUT_MULTILINE_BREAK_ON_WHITESPACE = '''Loaded plugins: langpacks, my plugin,
              : subscription-manager, product-id
Usage: yum [options] COMMAND
'''


def assert_plugins_identified_as_enabled(expected_plugins, identified_plugins):
    fail_description = 'Failed to parse a plugin from the package manager output.'
    for expected_enabled_plugin in expected_plugins:
        assert expected_enabled_plugin in identified_plugins, fail_description


@pytest.mark.parametrize(
    ('source_major_version', 'command'),
    [
        ('7', ['yum', '--setopt=debuglevel=2']),
        ('8', ['dnf', '-v']),
    ]
)
def test_scan_enabled_plugins(monkeypatch, source_major_version, command):
    """Tests whether the enabled plugins are correctly retrieved from the package manager output."""

    def run_mocked(cmd, **kwargs):
        if cmd == command:
            return {
                'stdout': CMD_YUM_OUTPUT.split('\n'),
                'stderr': 'You need to give some command',
                'exit_code': 1
            }
        raise ValueError('Tried to run an unexpected command.')

    def get_source_major_version_mocked():
        return source_major_version

    # The library imports `run` all the way into its namespace (from ...stdlib import run),
    # we must overwrite it there then:
    monkeypatch.setattr(pluginscanner, 'run', run_mocked)
    monkeypatch.setattr(pluginscanner, 'get_source_major_version', get_source_major_version_mocked)

    enabled_plugins = pluginscanner.scan_enabled_package_manager_plugins()
    assert_plugins_identified_as_enabled(
        ['langpacks', 'my plugin', 'subscription-manager', 'product-id'],
        enabled_plugins
    )


@pytest.mark.parametrize(
    ('yum_output',),
    [
        (CMD_YUM_OUTPUT,),
        (CMD_YUM_OUTPUT_MULTILINE_BREAK_ON_HYPHEN,),
        (CMD_YUM_OUTPUT_MULTILINE_BREAK_ON_WHITESPACE,)
    ])
def test_yum_loaded_plugins_multiline_output(yum_output, monkeypatch):
    """Tests whether the library correctly handles plugins getting reported on multiple lines."""
    def run_mocked(cmd, **kwargs):
        return {
            'stdout': yum_output.split('\n'),
            'stderr': 'You need to give some command',
            'exit_code': 1
        }

    monkeypatch.setattr(pluginscanner, 'run', run_mocked)
    monkeypatch.setattr(pluginscanner, 'get_source_major_version', lambda: '7')

    enabled_plugins = pluginscanner.scan_enabled_package_manager_plugins()

    assert len(enabled_plugins) == 4, 'Identified more yum plugins than available in the mocked yum output.'
    assert_plugins_identified_as_enabled(
        ['langpacks', 'my plugin', 'subscription-manager', 'product-id'],
        enabled_plugins
    )



from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api
from leapp.models import FIPSInfo


def read_sys_fips_state():
    with open('/proc/sys/crypto/fips_enabled') as fips_status_handle:
        return fips_status_handle.read().strip()


def check_fips_state_perserved():
    fips_info = next(api.consume(FIPSInfo), None)
    if not fips_info:
        # Unexpected, FIPSInfo is produced unconditionally
        raise StopActorExecutionError('Cannot check for the correct FIPS state in the upgrade initramfs',
                                      details={'Problem': 'Did not receive any FIPSInfo message'})

    if fips_info.is_enabled:
        fips_status = read_sys_fips_state()
        if fips_status != '1':
            details = {'details': ('The system is reporting FIPS as disabled, although it should be enabled'
                                   ' since it was enabled on the source system.')}
            raise StopActorExecutionError('Failed to enable FIPS in the upgrade initramfs', details=details)



import functools
import itertools
import json
import os
import shutil
import tarfile
from datetime import datetime

from leapp.cli.commands import command_utils
from leapp.cli.commands.config import get_config
from leapp.exceptions import CommandError
from leapp.repository.scan import find_and_scan_repositories
from leapp.utils import audit
from leapp.utils.audit import get_checkpoints, get_connection, get_messages
from leapp.utils.output import report_unsupported
from leapp.utils.report import fetch_upgrade_report_messages, generate_report_file


def disable_database_sync():
    def disable_db_sync_decorator(f):
        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            saved = os.environ.get('LEAPP_DEVEL_DATABASE_SYNC_OFF', None)
            try:
                os.environ['LEAPP_DEVEL_DATABASE_SYNC_OFF'] = '1'
                return f(*args, **kwargs)
            finally:
                os.environ.pop('LEAPP_DEVEL_DATABASE_SYNC_OFF')
                if saved:
                    os.environ['LEAPP_DEVEL_DATABASE_SYNC_OFF'] = saved
        return wrapper

    if not os.environ.get('LEAPP_DATABASE_FORCE_SYNC_ON', None):
        audit.create_connection = disable_db_sync_decorator(audit.create_connection)


def restore_leapp_env_vars(context):
    """
    Restores leapp environment variables from the `IPUConfig` message.
    """
    messages = get_messages(('IPUConfig',), context)
    leapp_env_vars = json.loads((messages or [{}])[0].get('message', {}).get('data', '{}')).get('leapp_env_vars', {})
    for entry in leapp_env_vars:
        os.environ[entry['name']] = entry['value']


def archive_logfiles():
    """ Archive log files from a previous run of Leapp """
    cfg = get_config()

    if not os.path.isdir(cfg.get('files_to_archive', 'dir')):
        os.makedirs(cfg.get('files_to_archive', 'dir'))

    files_to_archive = [os.path.join(cfg.get('files_to_archive', 'dir'), f)
                        for f in cfg.get('files_to_archive', 'files').split(',')
                        if os.path.isfile(os.path.join(cfg.get('files_to_archive', 'dir'), f))]

    if not os.path.isdir(cfg.get('archive', 'dir')):
        os.makedirs(cfg.get('archive', 'dir'))

    if files_to_archive:
        if os.path.isdir(cfg.get('debug', 'dir')):
            files_to_archive.append(cfg.get('debug', 'dir'))

        now = datetime.now().strftime('%Y%m%d%H%M%S')
        archive_file = os.path.join(cfg.get('archive', 'dir'), 'leapp-{}-logs.tar.gz'.format(now))

        with tarfile.open(archive_file, "w:gz") as tar:
            for file_to_add in files_to_archive:
                tar.add(file_to_add)
                if os.path.isdir(file_to_add):
                    shutil.rmtree(file_to_add, ignore_errors=True)
                try:
                    os.remove(file_to_add)
                except OSError:
                    pass
            # leapp_db is not in files_to_archive to not have it removed
            if os.path.isfile(cfg.get('database', 'path')):
                tar.add(cfg.get('database', 'path'))


def load_repositories_from(name, repo_path, manager=None):
    if get_config().has_option('repositories', name):
        repo_path = get_config().get('repositories', name)
    return find_and_scan_repositories(repo_path, manager=manager)


def load_repositories():
    manager = load_repositories_from('repo_path', '/etc/leapp/repo.d/', manager=None)
    manager.load()
    return manager


def fetch_last_upgrade_context(use_context=None):
    """
    :return: Context of the last execution
    """
    with get_connection(None) as db:
        if use_context:
            cursor = db.execute(
                "SELECT context, stamp, configuration FROM execution WHERE context = ?", (use_context,))
        else:
            cursor = db.execute(
                "SELECT context, stamp, configuration FROM execution WHERE kind = 'upgrade' ORDER BY id DESC LIMIT 1")
        row = cursor.fetchone()
        if row:
            return row[0], json.loads(row[2])
    return None, {}


def fetch_all_upgrade_contexts():
    """
    :return: All upgrade execution contexts
    """
    with get_connection(None) as db:
        cursor = db.execute(
            "SELECT context, stamp, configuration FROM execution WHERE kind = 'upgrade' ORDER BY id DESC")
        row = cursor.fetchall()
        if row:
            return row
    return None


def get_last_phase(context):
    checkpoints = get_checkpoints(context=context)
    if checkpoints:
        return checkpoints[-1]['phase']
    return None


def check_env_and_conf(env_var, conf_var, configuration):
    """
    Checks whether the given environment variable or the given configuration value are set to '1'
    """
    return os.getenv(env_var, '0') == '1' or configuration.get(conf_var, '0') == '1'


def generate_report_files(context, report_schema):
    """
    Generates all report files for specific leapp run (txt and json format)
    """
    cfg = get_config()
    report_txt, report_json = [os.path.join(cfg.get('report', 'dir'),
                                            'leapp-report.{}'.format(f)) for f in ['txt', 'json']]
    # fetch all report messages as a list of dicts
    messages = fetch_upgrade_report_messages(context)
    generate_report_file(messages, context, report_txt, report_schema)
    generate_report_file(messages, context, report_json, report_schema)


def get_cfg_files(section, cfg, must_exist=True):
    """
    Provide files from particular config section
    """
    files = []
    for file_ in cfg.get(section, 'files').split(','):
        file_path = os.path.join(cfg.get(section, 'dir'), file_)
        if not must_exist or must_exist and os.path.isfile(file_path):
            files.append(file_path)
    return files


def warn_if_unsupported(configuration):
    env = os.environ
    if env.get('LEAPP_UNSUPPORTED', '0') == '1':
        devel_vars = {k: env[k] for k in env if k.startswith('LEAPP_DEVEL_')}
        report_unsupported(devel_vars, configuration["whitelist_experimental"])


def handle_output_level(args):
    """
    Set environment variables following command line arguments.
    """
    os.environ['LEAPP_DEBUG'] = '1' if args.debug else os.getenv('LEAPP_DEBUG', '0')
    if os.environ['LEAPP_DEBUG'] == '1' or args.verbose:
        os.environ['LEAPP_VERBOSE'] = '1'
    else:
        os.environ['LEAPP_VERBOSE'] = os.getenv('LEAPP_VERBOSE', '0')


# NOTE(ivasilev) Please make sure you are not calling prepare_configuration after first reboot.
# If called as leapp upgrade --resume this will happily crash in target version container for
# the latest supported release because of target_version discovery attempt.
def prepare_configuration(args):
    """Returns a configuration dict object while setting a few env vars as a side-effect"""
    if args.whitelist_experimental:
        args.whitelist_experimental = list(itertools.chain(*[i.split(',') for i in args.whitelist_experimental]))
        os.environ['LEAPP_EXPERIMENTAL'] = '1'
    else:
        os.environ['LEAPP_EXPERIMENTAL'] = '0'
    os.environ['LEAPP_UNSUPPORTED'] = '0' if os.getenv('LEAPP_UNSUPPORTED', '0') == '0' else '1'
    if args.no_rhsm:
        os.environ['LEAPP_NO_RHSM'] = '1'
    elif not os.path.exists('/usr/sbin/subscription-manager'):
        os.environ['LEAPP_NO_RHSM'] = '1'
    elif os.getenv('LEAPP_NO_RHSM') != '1':
        os.environ['LEAPP_NO_RHSM'] = os.getenv('LEAPP_DEVEL_SKIP_RHSM', '0')

    if args.no_insights_register:
        os.environ['LEAPP_NO_INSIGHTS_REGISTER'] = '1'

    if args.enablerepo:
        os.environ['LEAPP_ENABLE_REPOS'] = ','.join(args.enablerepo)

    if os.environ.get('LEAPP_NO_RHSM', '0') == '1' or args.no_rhsm_facts:
        os.environ['LEAPP_NO_RHSM_FACTS'] = '1'

    if args.channel:
        os.environ['LEAPP_TARGET_PRODUCT_CHANNEL'] = args.channel

    if args.iso:
        os.environ['LEAPP_TARGET_ISO'] = args.iso
    target_iso_path = os.environ.get('LEAPP_TARGET_ISO')
    if target_iso_path:
        # Make sure we convert rel paths into abs ones while we know what CWD is
        os.environ['LEAPP_TARGET_ISO'] = os.path.abspath(target_iso_path)

    if args.nogpgcheck:
        os.environ['LEAPP_NOGPGCHECK'] = '1'

    # Check upgrade path and fail early if it's unsupported
    target_version, flavor = command_utils.vet_upgrade_path(args)
    os.environ['LEAPP_UPGRADE_PATH_TARGET_RELEASE'] = target_version
    os.environ['LEAPP_UPGRADE_PATH_FLAVOUR'] = flavor

    current_version = command_utils.get_os_release_version_id('/etc/os-release')
    os.environ['LEAPP_IPU_IN_PROGRESS'] = '{source}to{target}'.format(
        source=command_utils.get_major_version(current_version),
        target=command_utils.get_major_version(target_version)
    )

    configuration = {
        'debug': os.getenv('LEAPP_DEBUG', '0'),
        'verbose': os.getenv('LEAPP_VERBOSE', '0'),
        'whitelist_experimental': args.whitelist_experimental or (),
    }
    return configuration


def process_whitelist_experimental(repositories, workflow, configuration, logger=None):
    for actor_name in configuration.get('whitelist_experimental', ()):
        actor = repositories.lookup_actor(actor_name)
        if actor:
            workflow.whitelist_experimental_actor(actor)
        else:
            msg = 'No such Actor: {}'.format(actor_name)
            if logger:
                logger.error(msg)
            raise CommandError(msg)



import pytest

from leapp import reporting
from leapp.libraries.actor import blsgrubcfgonppc64
from leapp.libraries.common import testutils
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import DefaultGrub, DefaultGrubInfo, GrubCfgBios


@pytest.mark.parametrize(
    'is_bls,bls_cfg_enabled,is_ppc64', (
        (True, True, True),
        (True, True, False),
        (True, False, True),
        (False, True, False),
        (False, False, False),
    )
)
def test_check_grub_bls_cfg_ppc64(monkeypatch, is_bls, bls_cfg_enabled, is_ppc64):

    grub_cfg_msg = GrubCfgBios(insmod_bls=is_bls)

    bls_cfg_enabled = DefaultGrubInfo(
        default_grub_info=[DefaultGrub(name='GRUB_ENABLE_BLSCFG', value='true')]
    )

    bls_cfg_not_enabled = DefaultGrubInfo(
        default_grub_info=[DefaultGrub(name='GRUB_ENABLE_BLSCFG', value='false')]
    )

    bls_cfg = bls_cfg_enabled if bls_cfg_enabled else bls_cfg_not_enabled

    arch = testutils.architecture.ARCH_PPC64LE if is_ppc64 else testutils.architecture.ARCH_X86_64

    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[grub_cfg_msg, bls_cfg], arch=arch))
    blsgrubcfgonppc64.process()

    if (
        not is_bls and
        is_ppc64 and
        bls_cfg_enabled
    ):
        assert reporting.create_report.called
        assert reporting.create_report.report_fields['title'] == 'TBA'
    else:
        assert not reporting.create_report.called



import os

import pytest

from leapp import reporting
from leapp.libraries.actor import checketcreleasever
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked, logger_mocked
from leapp.libraries.stdlib import api
from leapp.models import (
    PkgManagerInfo,
    Report,
    RHUIInfo,
    TargetRHUIPostInstallTasks,
    TargetRHUIPreInstallTasks,
    TargetRHUISetupInfo
)


@pytest.mark.parametrize('exists', [True, False])
def test_etc_releasever(monkeypatch, exists):
    pkg_mgr_msg = [PkgManagerInfo(etc_releasever='7.7')] if exists else []
    expected_rel_ver = '6.10'

    mocked_report = create_report_mocked()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=pkg_mgr_msg, dst_ver=expected_rel_ver
        )
    )
    monkeypatch.setattr(reporting, 'create_report', mocked_report)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    checketcreleasever.process()

    if exists:
        assert reporting.create_report.called == 1
        assert expected_rel_ver in mocked_report.report_fields['summary']
        assert not api.current_logger.dbgmsg
    else:
        assert not reporting.create_report.called
        assert api.current_logger.dbgmsg


def test_etc_releasever_empty(monkeypatch):
    pkg_mgr_msg = [PkgManagerInfo(etc_releasever=None)]
    expected_rel_ver = '6.10'

    mocked_report = create_report_mocked()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=pkg_mgr_msg, dst_ver=expected_rel_ver
        )
    )
    monkeypatch.setattr(reporting, 'create_report', mocked_report)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    checketcreleasever.process()

    assert not reporting.create_report.called
    assert api.current_logger.dbgmsg


def mk_rhui_info():
    preinstall_tasks = TargetRHUIPreInstallTasks()
    postinstall_tasks = TargetRHUIPostInstallTasks()
    setup_info = TargetRHUISetupInfo(preinstall_tasks=preinstall_tasks, postinstall_tasks=postinstall_tasks)
    rhui_info = RHUIInfo(provider='aws',
                         src_client_pkg_names=['rh-amazon-rhui-client'],
                         target_client_pkg_names=['rh-amazon-rhui-client'],
                         target_client_setup_info=setup_info)
    return rhui_info


@pytest.mark.parametrize('is_rhui', [True, False])
def test_etc_releasever_rhui(monkeypatch, is_rhui):
    if is_rhui:
        rhui_msg = [mk_rhui_info()]
    else:
        rhui_msg = []

    expected_rel_ver = '6.10'

    mocked_report = create_report_mocked()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=rhui_msg, dst_ver=expected_rel_ver
        )
    )
    monkeypatch.setattr(reporting, 'create_report', mocked_report)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    checketcreleasever.process()

    if is_rhui:
        assert reporting.create_report.called == 1
        assert expected_rel_ver in mocked_report.report_fields['summary']
        assert not api.current_logger.dbgmsg
    else:
        assert not reporting.create_report.called
        assert api.current_logger.dbgmsg


def test_etc_releasever_neither(monkeypatch):
    mocked_report = create_report_mocked()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked())
    monkeypatch.setattr(reporting, 'create_report', mocked_report)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    checketcreleasever.process()

    assert not reporting.create_report.called
    assert api.current_logger.dbgmsg


def test_etc_releasever_both(monkeypatch):
    rhui_info = mk_rhui_info()

    msgs = [rhui_info, PkgManagerInfo(etc_releasever='7.7')]
    expected_rel_ver = '6.10'

    mocked_report = create_report_mocked()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=msgs, dst_ver=expected_rel_ver
        )
    )
    monkeypatch.setattr(reporting, 'create_report', mocked_report)
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    checketcreleasever.process()

    assert reporting.create_report.called == 1
    assert expected_rel_ver in mocked_report.report_fields['summary']
    assert not api.current_logger.dbgmsg



import pytest

# from leapp import reporting
from leapp.libraries.actor import checkdeprecatedrpmsignature

# from leapp.libraries.common.testutils import CurrentActorMocked
# from leapp.libraries.stdlib import api


@pytest.mark.parametrize('curr_state,expected_res', (
    ('LEGACY', True),
    ('DEFAULT:SHA1', True),
    ('MYPOL:SHA1', False),
    ('DEFAULT', False),
    ('NOLEGACY', False),
    ('SOMETHING:SHA-512', False),
    ('SOMETHING:-SHA1', False),
    ('DEFAULT:-SHA1', False),
))
def test_sha_allowed(curr_state, expected_res):
    assert expected_res == checkdeprecatedrpmsignature._is_sha1_allowed(curr_state)

# TODO: def test_get_rpms_sha1_sig()
# TODO: test_inhibitor_create



import re

from leapp.libraries.common import multipathutil

_bool_options = {'retain_attached_hw_handler': True, 'detect_prio': True,
                 'detect_path_checker': True, 'reassign_maps': False}

_exist_options = ('hw_str_match', 'ignore_new_boot_devs',
                  'new_bindings_in_boot', 'unpriv_sgio')

_ovr_options = ('hardware_handler', 'pg_timeout')


class _QueueIfNoPathInfo(object):
    def __init__(self, line, value):
        self.line = line
        self.value = value
        self.has_no_path_retry = False


def _nothing_to_do(config):
    if config.default_path_checker and config.default_path_checker != 'tur':
        return False

    config_checks = (
        (config.default_retain_hwhandler, False),
        (config.default_detect_prio, False),
        (config.default_detect_checker, False),
        (config.reassign_maps, True),
        (config.hw_str_match_exists, True),
        (config.ignore_new_boot_devs_exists, True),
        (config.new_bindings_in_boot_exists, True),
        (config.unpriv_sgio_exists, True),
        (config.detect_path_checker_exists, True),
        (config.overrides_hwhandler_exists, True),
        (config.overrides_pg_timeout_exists, True),
        (config.queue_if_no_path_exists, True),
        (config.all_devs_section_exists, True)
    )
    for option, value in config_checks:
        if option is value:
            return False

    return config.all_devs_options == []


def _comment_out_line(line):
    return '# ' + line + ' # Commented out by Leapp'


def _comment_out_ranges(lines, ranges):
    for start, end in ranges:
        line = lines[start]
        lines[start] = '# ' + line + ' # Section commented out by Leapp'
        for i in range(start + 1, end):
            line = lines[i]
            if line == '':
                lines[i] = '#'
            elif line[0] != '#':
                lines[i] = '# ' + line


def _setup_value(value):
    if re.search(r'\s', value):
        return '"' + value + '"'
    return value


def _add_overrides(lines, options):
    lines.append('overrides { # Section added by Leapp')
    for option in options:
        lines.append('\t{} {}'.format(option.name, _setup_value(option.value)))
    lines.append('}')
    lines.append('')


def _update_overrides(lines, ovr_line, options):
    new_lines = []
    start = None
    for i, line in enumerate(lines):
        if line is ovr_line:
            start = i + 1
            break
    if not start:
        return
    for option in options:
        new_lines.append('\t{} {} # Line added by Leapp'.
                         format(option.name, _setup_value(option.value)))
    lines[start:start] = new_lines      # insert new_lines


def _convert_checker_line(line):
    return line.replace('detect_path_checker', 'detect_checker') + \
            ' # Line modified by Leapp'


def _modify_features_line(line, value):
    items = value.split()
    if items == [] or not items[0].isdigit():
        return _comment_out_line(line)
    nr_features = int(items[0])
    if nr_features != len(items) - 1:
        return _comment_out_line(line)
    r = re.match('^(.*)features', line)
    if not r:
        return _comment_out_line(line)
    line_start = r.group(1)
    try:
        items.remove('queue_if_no_path')
    except ValueError:
        return _comment_out_line(line)
    items[0] = str(nr_features - 1)
    return line_start + 'features "' + ' '.join(items) + \
        '" # Line modified by Leapp'


def _add_npr(lines, line, i):
    r = re.match('^(.*)features', line)
    if not r:
        return
    line_start = r.group(1)
    lines.insert(i,
                 line_start + 'no_path_retry queue # Line added by Leapp')


def _remove_qinp(lines, qinp_infos):
    infos_iter = iter(qinp_infos)
    info = next(infos_iter, None)
    if not info:
        return
    i = 0
    while i < len(lines):
        if lines[i] is info.line:
            lines[i] = _modify_features_line(info.line, info.value)
            if not info.has_no_path_retry:
                _add_npr(lines, info.line, i + 1)
            info = next(infos_iter, None)
            if not info:
                return
        i += 1


def _valid_npr(value):
    if value.isdigit() and int(value) >= 0:
        return True
    if value in ('fail', 'queue'):
        return True
    return False


def _update_config(config):
    if _nothing_to_do(config):
        return None
    contents = multipathutil.read_config(config.pathname)
    if contents is None:
        return None
    lines = contents.split('\n')
    section = None
    in_subsection = False
    in_all_devs = False
    subsection_start = None
    all_devs_ranges = []
    overrides_line = None
    qinp_info = None
    has_no_path_retry = False
    qinp_infos = []
    for i, line in enumerate(lines):
        try:
            data = multipathutil.LineData(line, section, in_subsection)
        except ValueError:
            continue
        if data.type == data.TYPE_SECTION_END:
            if qinp_info and not in_all_devs:
                qinp_info.has_no_path_retry = has_no_path_retry
                qinp_infos.append(qinp_info)
            qinp_info = None
            has_no_path_retry = False
            if in_subsection:
                in_subsection = False
                if in_all_devs:
                    all_devs_ranges.append((subsection_start, i + 1))
                in_all_devs = False
                subsection_start = None
            elif section is not None:
                section = None
        elif data.type == data.TYPE_SECTION_START:
            if section is None:
                section = data.section
                if section == 'overrides':
                    overrides_line = line
            elif not in_subsection:
                in_subsection = True
                subsection_start = i
        if data.type != data.TYPE_OPTION:
            continue
        if section == 'defaults':
            if (data.option in ('path_checker', 'checker')) and data.value != 'tur':
                lines[i] = _comment_out_line(line)
                continue
            if data.option in _bool_options and \
                    _bool_options[data.option] != data.is_enabled():
                lines[i] = _comment_out_line(line)
                continue
        elif section == 'overrides' and data.option in _ovr_options:
            lines[i] = _comment_out_line(line)
            continue
        elif section == 'devices' and in_subsection and \
                data.option == 'all_devs' and data.is_enabled():
            in_all_devs = True
            continue
        if data.option in _exist_options:
            lines[i] = _comment_out_line(line)
        elif data.option == 'detect_path_checker':
            lines[i] = _convert_checker_line(line)
        elif data.option == 'no_path_retry' and _valid_npr(data.value):
            has_no_path_retry = True
        elif data.option == 'features' and 'queue_if_no_path' in data.value:
            qinp_info = _QueueIfNoPathInfo(line, data.value)

    if in_subsection:
        lines.append('\t} # line added by Leapp')
        if in_all_devs:
            all_devs_ranges.append((subsection_start, len(lines)))
        elif qinp_info:
            qinp_info.has_no_path_retry = has_no_path_retry
            qinp_infos.append(qinp_info)
            qinp_info = None
    if section is not None:
        lines.append('} # line added by Leapp')
        lines.append('')
        if qinp_info:
            qinp_info.has_no_path_retry = has_no_path_retry
            qinp_infos.append(qinp_info)
    _comment_out_ranges(lines, all_devs_ranges)
    if qinp_infos:
        _remove_qinp(lines, qinp_infos)
    if config.all_devs_options != []:
        if overrides_line:
            _update_overrides(lines, overrides_line, config.all_devs_options)
        else:
            _add_overrides(lines, config.all_devs_options)
    contents = '\n'.join(lines)
    return contents


def update_configs(facts):
    for config in facts.configs:
        contents = _update_config(config)
        if contents:
            multipathutil.write_config(config.pathname, contents)



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class FirewalldFacts(Model):
    """The model contains firewalld configuration."""
    topic = SystemInfoTopic

    firewall_config_command = fields.String(default='')
    ebtablesTablesInUse = fields.List(fields.String(), default=[])
    ipsetTypesInUse = fields.List(fields.String(), default=[])



from leapp.actors import Actor
from leapp.libraries.actor.quaggatofrr import process_facts
from leapp.models import QuaggaToFrrFacts
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag


class QuaggaToFrr(Actor):
    """
    Edit frr configuration on the new system.

    Take gathered info about quagga from RHEL 7 and apply these to frr in RHEL 8.
    """

    name = 'quagga_to_frr'
    consumes = (QuaggaToFrrFacts, )
    produces = ()
    tags = (ApplicationsPhaseTag, IPUWorkflowTag)

    def process(self):
        quagga_facts = next(self.consume(QuaggaToFrrFacts), None)

        if quagga_facts:
            process_facts(quagga_facts)



from leapp.actors import Actor
from leapp.libraries.actor.rootscanner import scan_dir
from leapp.models import RootDirectory
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class RootScanner(Actor):
    """
    Scan the system root directory and produce a message containing
    information about its subdirectories.
    """

    name = 'root_scanner'
    consumes = ()
    produces = (RootDirectory,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        self.produce(scan_dir(b'/'))



from leapp.actors import Actor
from leapp.libraries.actor import checktargetrepos
from leapp.models import CustomTargetRepositoryFile, Report, RHUIInfo, TargetRepositories
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class Checktargetrepos(Actor):
    """
    Check whether target yum repositories are specified.

    RHSM | RHUI | ER | CTR | CTRF || result
    -----+------+----+-----+------++-------
     Yes | ---- | -- | --- | ---- || -
    -----+------+----+-----+------++-------
     No  | Yes  | -- | --- | ---- || -
    -----+------+----+-----+------++-------
     No  | No   | -- | No  | No   || inhibit
    -----+------+----+-----+------++-------
     No  | No   | -- | No  | Yes  || inhibit
    -----+------+----+-----+------++-------
     No  | No   | No | Yes | No   || warn/report info
    -----+------+----+-----+------++-------
     No  | No   | No | Yes | Yes  || -
    -----+------+----+-----+------++-------
     No  | No   | Yes| Yes | No   || -
    -----+------+----+-----+------++-------
     No  | No   | Yes| Yes | Yes  || -

       ER   - config.get_env('LEAPP_ENABLE_REPOS') is non-empty
       CTR  - CustomTargetRepositories found
       CTRF - the expected CustomTargetRepositoryFile found
       RHSM - RHSM is used (it is not skipped)
       RHUI - RHUIInfo msg is consumed

    This is not 100 % reliable check. This cover just the most obvious cases
    that are expected to fail. Reporting of such issues in this way, here,
    will be probably much more clear, without additional errors that could
    be raised.
    """

    name = 'checktargetrepos'
    consumes = (CustomTargetRepositoryFile, TargetRepositories, RHUIInfo)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        checktargetrepos.process()



import pytest
from mock import Mock, patch

from leapp.libraries.actor import cephvolumescan
from leapp.models import InstalledRPM, LsblkEntry, RPM, StorageInfo
from leapp.reporting import Report

CONT_PS_COMMAND_OUTPUT = {
    "stdout":
    """CONTAINER ID IMAGE COMMAND CREATED STATUS PORTS NAMES
    50d96fe72019 registry.redhat.io/rhceph/rhceph-4-rhel8:latest "/opt/ceph-contain..." \
        2 weeks ago Up 2 weeks ceph-osd-0
    f93c17b49c40 registry.redhat.io/rhceph/rhceph-4-rhel8:latest "/opt/ceph-contain..." \
        2 weeks ago Up 2 weeks ceph-osd-1
    0669880c52dc registry.redhat.io/rhceph/rhceph-4-rhel8:latest "/opt/ceph-contain..." \
        2 weeks ago Up 2 weeks ceph-mgr-ceph4-standalone
    d7068301294e registry.redhat.io/rhceph/rhceph-4-rhel8:latest "/opt/ceph-contain..." \
        2 weeks ago Up 2 weeks ceph-mon-ceph4-standalone
    63de6d00f241 registry.redhat.io/openshift4/ose-prometheus-alertmanager:4.1 "/bin/alertmanager..." \
        2 weeks ago Up 2 weeks alertmanager
    28ed65960c80 registry.redhat.io/rhceph/rhceph-4-dashboard-rhel8:4 "/run.sh" \
        2 weeks ago Up 2 weeks grafana-server
    f4b300d7a11f registry.redhat.io/openshift4/ose-prometheus-node-exporter:v4.1 "/bin/node_exporte..." \
        2 weeks ago Up 2 weeks node-exporter
    95a03700b3ff registry.redhat.io/openshift4/ose-prometheus:4.1 "/bin/prometheus -..." \
        2 weeks ago Up 2 weeks prometheus"""
    }

CEPH_VOLUME_OUTPUT = {
    "stdout": """{
        "0":[
            {
                "devices":[
                    "/dev/sda"
                ],
                "lv_name":"osd-block-c5215ba7-517b-45c7-88df-37a03eeaa0e9",
                "lv_uuid":"Tyc0TH-RDxr-ebAF-9mWF-Kh5R-YnvJ-cEcGVn",
                "tags":{
                    "ceph.encrypted":"1"
                },
                "type":"block",
                "vg_name":"ceph-a696c40d-6b1d-448d-a40e-fadca22b64bc"
            }
        ]
    }"""
}

CEPH_LVM_LIST = {
    '0': [{'devices': ['/dev/sda'],
           'lv_name': 'osd-block-c5215ba7-517b-45c7-88df-37a03eeaa0e9',
           'lv_uuid': 'Tyc0TH-RDxr-ebAF-9mWF-Kh5R-YnvJ-cEcGVn',
           'tags': {'ceph.encrypted': '1'},
           'type': 'block',
           'vg_name': 'ceph-a696c40d-6b1d-448d-a40e-fadca22b64bc'}]
    }


@patch('leapp.libraries.actor.cephvolumescan.run')
def test_select_osd_container(m_run):

    m_run.return_value = CONT_PS_COMMAND_OUTPUT

    assert cephvolumescan.select_osd_container('docker') == "ceph-osd-0"


@patch('leapp.libraries.actor.cephvolumescan.has_package')
@patch('leapp.libraries.actor.cephvolumescan.select_osd_container')
@patch('leapp.libraries.actor.cephvolumescan.run')
def test_get_ceph_lvm_list(m_run, m_osd_container, m_has_package):

    m_has_package.return_value = True
    m_osd_container.return_value = 'podman'
    m_run.return_value = CEPH_VOLUME_OUTPUT

    assert cephvolumescan.get_ceph_lvm_list() == CEPH_LVM_LIST


@patch('leapp.libraries.actor.cephvolumescan.os.path.isfile')
@patch('leapp.libraries.actor.cephvolumescan.get_ceph_lvm_list')
def test_encrypted_osds_list(m_get_ceph_lvm_list, m_isfile):

    m_get_ceph_lvm_list.return_value = CEPH_LVM_LIST
    m_isfile.return_value = True

    assert cephvolumescan.encrypted_osds_list() == ['Tyc0TH-RDxr-ebAF-9mWF-Kh5R-YnvJ-cEcGVn']



import json
import os

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common import rhui
from leapp.libraries.common.config import get_env
from leapp.libraries.stdlib import api
from leapp.models import DistributionSignedRPM, InstalledRedHatSignedRPM, InstalledRPM, InstalledUnsignedRPM


def get_distribution_data(distribution):
    distributions_path = api.get_common_folder_path('distro')

    distribution_config = os.path.join(distributions_path, distribution, 'gpg-signatures.json')
    if os.path.exists(distribution_config):
        with open(distribution_config) as distro_config_file:
            distro_config_json = json.load(distro_config_file)
            distro_keys = distro_config_json.get('keys', [])
            # distro_packager = distro_config_json.get('packager', 'not-available')
    else:
        raise StopActorExecutionError(
            'Cannot find distribution signature configuration.',
            details={'Problem': 'Distribution {} was not found in {}.'.format(distribution, distributions_path)})
    return distro_keys


def is_distro_signed(pkg, distro_keys):
    return any(key in pkg.pgpsig for key in distro_keys)


def is_exceptional(pkg, allowlist):
    """
    Some packages should be marked always as signed

    tl;dr; gpg-pubkey, katello packages, and rhui packages

    gpg-pubkey is not real RPM. It's just an entry representing
    gpg key imported inside the RPM DB. For that same reason, it cannot be
    signed. Note that it cannot affect the upgrade transaction, so ignore
    who vendored the key. Total majority of all machines have imported third
    party gpg keys.

    Katello packages have various names and are created on a Satellite server.

    The allowlist is now used for any other package names that should be marked
    always as signed for the particular upgrade.
    """
    return pkg.name == 'gpg-pubkey' or pkg.name.startswith('katello-ca-consumer') or pkg.name in allowlist


def process():
    distribution = api.current_actor().configuration.os_release.release_id
    distro_keys = get_distribution_data(distribution)
    all_signed = get_env('LEAPP_DEVEL_RPMS_ALL_SIGNED', '0') == '1'
    rhui_pkgs = rhui.get_all_known_rhui_pkgs_for_current_upg()

    signed_pkgs = DistributionSignedRPM()
    rh_signed_pkgs = InstalledRedHatSignedRPM()
    unsigned_pkgs = InstalledUnsignedRPM()

    for rpm_pkgs in api.consume(InstalledRPM):
        for pkg in rpm_pkgs.items:
            if all_signed or is_distro_signed(pkg, distro_keys) or is_exceptional(pkg, rhui_pkgs):
                signed_pkgs.items.append(pkg)
                if distribution == 'rhel':
                    rh_signed_pkgs.items.append(pkg)
                continue
            unsigned_pkgs.items.append(pkg)

    api.produce(signed_pkgs)
    api.produce(rh_signed_pkgs)
    api.produce(unsigned_pkgs)



from leapp.actors import Actor
from leapp.libraries.actor import pcidevicesscanner
from leapp.models import DetectedDeviceOrDriver, DeviceDriverDeprecationData, PCIDevices
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class PCIDevicesScanner(Actor):
    """
    Provides data about existing PCI Devices.

    After collecting data from lspci, a message with relevant data will be produced.
    """

    name = 'pci_devices_scanner'
    consumes = (DeviceDriverDeprecationData,)
    produces = (PCIDevices, DetectedDeviceOrDriver)
    tags = (IPUWorkflowTag, FactsPhaseTag,)

    def process(self):
        pcidevicesscanner.scan_pci_devices(self.produce)



from leapp import reporting
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common.rpms import has_package
from leapp.libraries.stdlib import api
from leapp.models import DistributionSignedRPM, NISConfig

report_summary = (
    'The NIS components (ypserv, ypbind, and yp-tools) are no longer available in RHEL-9.'
    ' The technology behind those packages is based an outdated design patterns, which are'
    ' no longer considered as secure. There is no direct alternative with fully compatible'
    ' features.'
)

report_hint = (
    'The alternatives are LDAP and for some use cases Kerberos or migrating to IPA.'
)

report_link_url = 'https://access.redhat.com/solutions/5991271'


def report_nis():
    """
    Create the report if any of NIS packages (RH signed)
    is installed and configured.

    Should notify user about present NIS component package
    installation, warn them about discontinuation, and
    redirect them to online documentation for possible
    alternatives.
    """

    installed_packages = []
    configured_rpms = []

    # Get necessary models created by another actors
    nis_confs = api.consume(NISConfig)
    nis_conf = next(nis_confs, None)
    if not nis_conf:
        raise StopActorExecutionError(
            'Could not obtain NIS RPM information',
            details={'details': 'Actor did not receive NISConfig message.'}
        )

    if next(nis_confs, None):
        api.current_logger().warning('Unexpectedly received more than one NISConfig message.')

    configured_rpms = nis_conf.nis_not_default_conf

    installed_packages = [package for package in (
        'ypserv', 'ypbind') if has_package(DistributionSignedRPM, package)]

    # Final list of NIS packages (configured and installed)
    rpms_configured_installed = [x for x in installed_packages if x in configured_rpms]

    if not rpms_configured_installed:
        return

    # Create report
    report_content = [
        reporting.Title('NIS component has been detected on your system'),
        reporting.Summary(report_summary),
        reporting.Severity(reporting.Severity.MEDIUM),
        reporting.Groups([reporting.Groups.SERVICES]),
        reporting.ExternalLink(title='RHEL 9 (NIS) discontinuation',
                               url=report_link_url),
        reporting.Remediation(hint=report_hint),
    ]

    related_resources = [reporting.RelatedResource('package', pkg) for pkg in rpms_configured_installed]
    report_content += related_resources

    reporting.create_report(report_content)



from leapp import reporting
from leapp.actors import Actor
from leapp.libraries.actor import private
from leapp.models import FirewalldFacts
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag

related = [reporting.RelatedResource('package', 'firewalld')]


class CheckFirewalld(Actor):
    """
    Check for certain firewalld configuration that may prevent an upgrade.
    """

    name = 'check_firewalld'
    consumes = (FirewalldFacts,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        unsupported_tables = []
        unsupported_ipset_types = []
        list_separator_fmt = '\n    -'
        for facts in self.consume(FirewalldFacts):
            for table in facts.ebtablesTablesInUse:
                if not private.isEbtablesTableSupported(table):
                    unsupported_tables.append(table)
            for ipset_type in facts.ipsetTypesInUse:
                if not private.isIpsetTypeSupportedByNftables(ipset_type):
                    unsupported_ipset_types.append(ipset_type)

        if unsupported_tables:
            format_tuple = (
                list_separator_fmt,
                list_separator_fmt.join(list(set(unsupported_tables))),)
            create_report([
                reporting.Title('Firewalld is using an unsupported ebtables table.'),
                reporting.Summary('ebtables in RHEL-8 does not support these tables:{}{}'.format(*format_tuple)),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Groups([
                        reporting.Groups.FIREWALL,
                        reporting.Groups.SECURITY,
                        reporting.Groups.NETWORK
                ]),
                reporting.Groups([
                        reporting.Groups.INHIBITOR
                ]),
                reporting.Remediation(
                    hint='Remove firewalld direct rules that use these ebtables tables:{}{}'.format(*format_tuple)
                )
            ] + related)

        if unsupported_ipset_types:
            format_tuple = (
                list_separator_fmt,
                list_separator_fmt.join(list(set(unsupported_ipset_types))),)
            create_report([
                reporting.Title('Firewalld is using an unsupported ipset type.'),
                reporting.Summary(
                    'These ipset types are not supported by firewalld\'s nftables backend:{}{}'.format(*format_tuple)
                ),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Groups([
                        reporting.Groups.FIREWALL,
                        reporting.Groups.SECURITY,
                        reporting.Groups.NETWORK
                ]),
                reporting.Groups([
                        reporting.Groups.INHIBITOR
                ]),
                reporting.Remediation(
                    hint='Remove ipsets of these types from firewalld:{}{}'.format(*format_tuple)
                )
            ] + related)



from leapp import reporting
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common.config.version import get_source_major_version, get_target_major_version
from leapp.libraries.stdlib import api
from leapp.models import CustomTargetRepository, RepositoriesBlacklisted, RepositoriesFacts, RepositoriesMapping

# {OS_MAJOR_VERSION: PESID}
UNSUPPORTED_PESIDS = {
    "7": "rhel7-optional",
    "8": "rhel8-CRB",
    "9": "rhel9-CRB"
}


def _report_using_unsupported_repos(repos):
    report = [
        reporting.Title("Using repository not supported by Red Hat"),
        reporting.Summary(
            "The following repositories have been used for the "
            "upgrade, but they are not supported by the Red Hat.:\n"
            "- {}".format("\n - ".join(repos))
        ),
        reporting.Severity(reporting.Severity.HIGH),
        reporting.Groups([reporting.Groups.REPOSITORY]),
    ]
    reporting.create_report(report)


def _report_excluded_repos(repos):
    optional_repository_name = 'optional' if get_source_major_version() == '7' else 'CRB'
    api.current_logger().info(
        "The {0} repository is not enabled. Excluding {1} from the upgrade".format(optional_repository_name, repos)
    )

    report = [
        reporting.Title("Excluded target system repositories"),
        reporting.Summary(
            "The following repositories are not supported by "
            "Red Hat and are excluded from the list of repositories "
            "used during the upgrade.\n- {}".format("\n- ".join(repos))
        ),
        reporting.Severity(reporting.Severity.INFO),
        reporting.Groups([reporting.Groups.REPOSITORY]),
        reporting.Groups([reporting.Groups.FAILURE]),
        reporting.Remediation(
            hint=(
                "If some of excluded repositories are still required to be used"
                " during the upgrade, execute leapp with the --enablerepo option"
                " with the repoid of the repository required to be enabled"
                " as an argument (the option can be used multiple times)."
            )
        ),
    ]
    reporting.create_report(report)


def _get_manually_enabled_repos():
    """
    Get a set of repositories (repoids) that are manually enabled.

    manually enabled means (
        specified by --enablerepo option of the leapp command or
        inside the /etc/leapp/files/leapp_upgrade_repositories.repo),
    )
    :rtype: set [repoid]
    """
    try:
        return {repo.repoid for repo in api.consume(CustomTargetRepository)}
    except StopIteration:
        return set()


def _get_pesid_repos(repo_mapping, pesid, major_version):
    """
    Returns a list of pesid repos with the specified pesid and major version.

    :param str pesid: The PES ID representing the family of repositories.
    :param str major_version: The major version of the RHEL OS.
    :returns: A set of repoids with the specified pesid and major version.
    :rtype: List[PESIDRepositoryEntry]
    """
    pesid_repos = []
    for pesid_repo in repo_mapping.repositories:
        if pesid_repo.pesid == pesid and pesid_repo.major_version == major_version:
            pesid_repos.append(pesid_repo)
    return pesid_repos


def _get_repoids_to_exclude(repo_mapping):
    """
    Returns a set of repoids that should be blacklisted on the target system.

    :param RepositoriesMapping repo_mapping: Repository mapping data.
    :returns: A set of repoids to blacklist on the target system.
    :rtype: Set[str]
    """
    pesid_repos_to_exclude = _get_pesid_repos(repo_mapping,
                                              UNSUPPORTED_PESIDS[get_target_major_version()],
                                              get_target_major_version())
    return {pesid_repo.repoid for pesid_repo in pesid_repos_to_exclude}


def _are_optional_repos_disabled(repo_mapping, repos_on_system):
    """
    Checks whether all optional repositories are disabled.

    :param RepositoriesMapping repo_mapping: Repository mapping data.
    :param RepositoriesFacts repos_on_system: Installed repositories on the source system.
    :returns: True if there are any optional repositories enabled on the source system.
    """

    # Get a set of all repo_ids that are optional
    optional_pesid_repos = _get_pesid_repos(repo_mapping,
                                            UNSUPPORTED_PESIDS[get_source_major_version()],
                                            get_source_major_version())

    optional_repoids = [optional_pesid_repo.repoid for optional_pesid_repo in optional_pesid_repos]

    # Gather all optional repositories on the source system that are not enabled
    for repofile in repos_on_system.repositories:
        for repository in repofile.data:
            if repository.repoid in optional_repoids and repository.enabled:
                return False
    return True


def process():
    """
    Exclude target repositories provided by Red Hat without support.

    Conditions to exclude:
    - there are not such repositories already enabled on the source system
      (e.g. "Optional" repositories)
    - such repositories are not required for the upgrade explicitly by the user
      (e.g. via the --enablerepo option or via the /etc/leapp/files/leapp_upgrade_repositories.repo file)

    E.g. CRB repository is provided by Red Hat but it is without the support.
    """

    repo_mapping = next(api.consume(RepositoriesMapping), None)
    repos_facts = next(api.consume(RepositoriesFacts), None)

    # Handle required messages not received
    missing_messages = []
    if not repo_mapping:
        missing_messages.append('RepositoriesMapping')
    if not repos_facts:
        missing_messages.append('RepositoriesFacts')
    if missing_messages:
        raise StopActorExecutionError('Actor didn\'t receive required messages: {0}'.format(
            ', '.join(missing_messages)
        ))

    if not _are_optional_repos_disabled(repo_mapping, repos_facts):
        # nothing to do - an optional repository is enabled
        return

    # Optional repos are either not present or they are present, but disabled -> blacklist them on target system
    repos_to_exclude = _get_repoids_to_exclude(repo_mapping)

    # Do not exclude repos manually enabled from the CLI
    manually_enabled_repos = _get_manually_enabled_repos() & repos_to_exclude
    filtered_repos_to_exclude = repos_to_exclude - manually_enabled_repos

    if manually_enabled_repos:
        _report_using_unsupported_repos(manually_enabled_repos)
    if filtered_repos_to_exclude:
        _report_excluded_repos(filtered_repos_to_exclude)
        api.produce(RepositoriesBlacklisted(repoids=list(filtered_repos_to_exclude)))



from leapp.libraries.actor import transactionworkarounds as tw
from leapp.models import RpmTransactionTasks


def test_transactionworkarounds_2rpms(monkeypatch):
    result = []
    monkeypatch.setattr(tw.os, 'listdir', lambda _: ('abc.rpm', 'def.rpm', 'random.file'),)
    monkeypatch.setattr(tw.api, 'get_folder_path', lambda _: '/FAKE/FOLDER/PATH')
    monkeypatch.setattr(tw.api, 'produce', lambda *models: result.extend(models))

    tw.process()

    assert result
    assert isinstance(result[0], RpmTransactionTasks)
    assert result[0].local_rpms
    assert result[0].local_rpms == ['/FAKE/FOLDER/PATH/abc.rpm', '/FAKE/FOLDER/PATH/def.rpm']
    assert not result[0].to_install
    assert not result[0].to_remove
    assert not result[0].to_keep


def test_transactionworkarounds_0rpms(monkeypatch):
    result = []
    monkeypatch.setattr(tw.os, 'listdir', lambda _: ('abc.not', 'def.duh', 'random.file'),)
    monkeypatch.setattr(tw.api, 'get_folder_path', lambda _: '/FAKE/FOLDER/PATH')
    monkeypatch.setattr(tw.api, 'produce', lambda *models: result.extend(models))
    tw.process()
    assert not result



import operator

import six

from leapp.libraries.common import kernel as kernel_lib
from leapp.libraries.stdlib import api
from leapp.utils.deprecation import deprecated

OP_MAP = {
    '>': operator.gt,
    '>=': operator.ge,
    '<': operator.lt,
    '<=': operator.le
}

_SUPPORTED_VERSIONS = {
    # Note: 'rhel-alt' is detected when on 'rhel' with kernel 4.x
    '7': {'rhel': ['7.9'], 'rhel-alt': [], 'rhel-saphana': ['7.9']},
    '8': {'rhel': ['8.8', '8.10'], 'rhel-saphana': ['8.8', '8.10']},
}


def get_major_version(version):
    """
    Return the major version from the given version string.

    Versioning schema: MAJOR.MINOR.PATCH
    It doesn't matter how many dots are present. Everything until the first dot is returned. E.g.:
       8.1.0 => 8
       7.9   => 7
       7     => 7

    :param str version: The version string according to the versioning schema described.
    :rtype: str
    :returns: The major version from the given version string.
    """
    return version.split('.')[0]


def get_source_version():
    """
    Return the version of the source system.

    :rtype: str
    :returns: The version of the source system.
    """
    return api.current_actor().configuration.version.source


def get_source_major_version():
    """
    Return the major version of the source (original) system.

    For more details about about the versioning schema see :func:`get_major_version`.

    :rtype: str
    :returns: The major version of the source system.
    """
    return get_major_version(get_source_version())


def get_target_version():
    """
    Return the version of the target system.

    :rtype: str
    :returns: The version of the target system.
    """
    return api.current_actor().configuration.version.target


def get_target_major_version():
    """
    Return the major version of the target system.

    For more details about about the versioning schema see :func:`get_major_version`.

    :rtype: str
    :returns: The major version of the target system.
    """
    return get_major_version(get_target_version())


class _SupportedVersionsDict(dict):
    """
    Class for _SUPPORTED_VERSIONS lazy evaluation until ipuworkflowconfig actor data
    is ready.
    """

    def __init__(self):  # pylint: disable=W0231
        self.data = {}

    def _feed_supported_versions(self):
        major = get_source_major_version()
        if major not in _SUPPORTED_VERSIONS:
            raise KeyError('{} is not a supported source version of RHEL'.format(major))
        self.data = _SUPPORTED_VERSIONS[major]

    def __getitem__(self, key):
        self._feed_supported_versions()
        return self.data[key]

    def __iter__(self):
        self._feed_supported_versions()
        for d in self.data:
            yield d

    def __repr__(self):
        self._feed_supported_versions()
        return repr(self.data)

    def __contains__(self, x):
        self._feed_supported_versions()
        return x in self.data

    def __len__(self):
        self._feed_supported_versions()
        return len(self.data)

    def __str__(self):
        self._feed_supported_versions()
        return str(self.data)


SUPPORTED_VERSIONS = _SupportedVersionsDict()


def _version_to_tuple(version):
    """Converts the version string ``major.minor`` to ``(major, minor)`` int tuple."""
    major, minor = version.split('.')
    return (int(major), int(minor))


def _validate_versions(versions):
    """Raise ``TypeError`` if provided versions are not strings in the form ``<integer>.<integer>``."""
    for ver in versions:
        split = ver.split('.')
        if not len(split) == 2 or not all(x.isdigit() for x in split):
            raise ValueError("Versions have to be in the form of '<integer>.<integer>' "
                             "but provided was '{}'".format(versions))


def _simple_versions(versions):
    """Return ``True`` if provided versions are list of strings without comparison operators."""
    return all(len(v.split()) == 1 for v in versions)


def _cmp_versions(versions):
    """Return ``True`` if provided versions are list of strings with comparison operators."""
    split = [v.split() for v in versions]
    if not all(len(s) == 2 for s in split):
        return False

    return all(s[0] in OP_MAP for s in split)


def matches_version(match_list, detected):
    """
    Check if the `detected` version meets the criteria specified in `match_list`.

    :param match_list: specification of versions to check against
    :type match_list: list or tuple of strings in one of the two following forms:
                      ``['>'|'>='|'<'|'<='] <integer>.<integer>`` form, where elements are ANDed,
                      meaning that ``['>= 7.6', '< 7.8']`` would match for ``'7.6'``, and ``'7,7'`` only.
                      ``<integer>.<integer>`` form, where elements are ORed, meaning that
                      ``['7.6', '7.7']`` would match for ``'7.6'``, and ``'7,7'`` only.
                      These two forms cannot be mixed, otherwise ``ValueError`` is raised.
    :param detected: detected version
    :type detected: string in the form ``<integer>.<integer>``
    :return: ``True`` if `detected` value matches one of the values in `match_list`, ``False`` otherwise
    :rtype: bool
    """
    if not isinstance(match_list, (list, tuple)):
        raise TypeError("Versions to check against have to be a list or tuple "
                        "but provided was {}: '{}'".format(type(match_list), match_list))
    if not all(isinstance(e, six.string_types) for e in match_list):
        raise TypeError("Versions to check against have to be a list or tuple of strings "
                        "but provided was {}: '{}'".format([type(e) for e in match_list], match_list))
    if not isinstance(detected, six.string_types):
        raise TypeError("Detected version has to be a string "
                        "but provided was {}: '{}'".format(type(detected), detected))
    _validate_versions([detected])

    if _simple_versions(match_list):
        # match_list = ['7.6', '7.7', '7.8', '7.9']
        _validate_versions(match_list)
        return detected in match_list
    if _cmp_versions(match_list):
        detected = _version_to_tuple(detected)
        # match_list = ['>= 7.6', '< 7.10']
        _validate_versions([s.split()[1] for s in match_list])
        for match in match_list:
            op, ver = match.split()
            ver = _version_to_tuple(ver)
            if not OP_MAP[op](detected, ver):
                return False
        return True

    raise ValueError("Versions have to be a list or tuple of strings in the form "
                     "'['>'|'>='|'<'|'<='] <integer>.<integer>' or "
                     "'<integer>.<integer>' but provided was '{}'".format(match_list))


def matches_source_version(*match_list):
    """
    Check if one of provided source versions matches the configured one.

    :param match_list: specification of versions to check against
    :type match_list: strings, for details see argument ``match_list`` of function :func:`matches_version`.
    """
    return matches_version(match_list, get_source_version())


def matches_target_version(*match_list):
    """
    Check if one of provided target versions matches the configured one.

    :param match_list: specification of versions to check against
    :type match_list: strings, for details see argument ``match_list`` of function :func:`matches_version`.
    """
    return matches_version(match_list, get_target_version())


def matches_release(allowed_releases, release):
    """
    Check if the given `release` is allowed to upgrade based in `allowed_releases`.

    :param allowed_releases: All supported releases
    :type allowed_releases: list or dict
    :param release: release name to be checked
    :type release: string
    :return: ``True`` if `release` value matches one of the values in `allowed_releases`, ``False`` otherwise
    :rtype: bool
    """
    if not (release and allowed_releases):
        return False

    return release in allowed_releases


def current_version():
    """
    Return the current Linux release and version.

    :return: The tuple contains release name and version value.
    :rtype: (string, string)
    """
    release = api.current_actor().configuration.os_release
    return release.release_id, release.version_id


def is_default_flavour():
    """
    Check if the current system uses the default upgrade path.

    :return: `True` if this upgrade process is using the default upgrade path and `False` otherwise.
    :rtype: bool
    """
    return api.current_actor().configuration.flavour == 'default'


def is_sap_hana_flavour():
    """
    Check if the current system needs to use the SAP HANA upgrade path.

    :return: `True` if this upgrade process is using the SAP HANA upgrade path and `False` otherwise.
    :rtype: bool
    """
    return api.current_actor().configuration.flavour == 'saphana'


def is_rhel_alt():
    """
    Check if the current system is RHEL-ALT or not (only for RHEL 7)

    The function is valid only for the RHEL 7 systems. On RHEL 8+ systems
    returns always False.

    :return: `True` if the current system is RHEL-ALT and `False` otherwise.
    :rtype: bool
    """

    if get_source_major_version() != '7':
        return False
    conf = api.current_actor().configuration
    # rhel-alt is rhel 7 with kernel 4.x - there is not better detection...
    return conf.os_release.release_id == 'rhel' and conf.kernel[0] == '4'


@deprecated(since='2023-08-15', message='This information is now provided by KernelInfo message.')
def is_rhel_realtime():
    """
    Check whether the original system is RHEL Real Time.

    Currently the check is based on the release of the original booted kernel.
    In case of RHEL, we are sure the release contains the ".rt" string and
    non-realtime kernels don't. Let's use this minimalistic check for now.
    In future, we could detect whether the system is preemptive or not based
    on properties of the kernel (e.g. uname -a tells that information).

    :return: `True` if the orig system is RHEL RT and `False` otherwise.
    :rtype: bool
    """
    conf = api.current_actor().configuration
    if conf.os_release.release_id != 'rhel':
        return False

    kernel_type = kernel_lib.determine_kernel_type_from_uname(get_source_version(), conf.kernel)
    return kernel_type == kernel_lib.KernelType.REALTIME


def is_supported_version():
    """
    Verify if the current system version is supported for the upgrade.

    :return: `True` if the current version is supported and `False` otherwise.
    :rtype: bool
    """
    release_id, version_id = current_version()
    if is_rhel_alt():
        release_id = 'rhel-alt'
    elif is_sap_hana_flavour():
        release_id = 'rhel-saphana'

    if not matches_release(SUPPORTED_VERSIONS, release_id):
        return False

    return matches_version(SUPPORTED_VERSIONS[release_id], version_id)



import os

import pytest

from leapp.libraries.actor import applycustomdnfconf


@pytest.mark.parametrize(
    "exists,should_move",
    [(False, False), (True, True)],
)
def test_copy_correct_dnf_conf(monkeypatch, exists, should_move):
    monkeypatch.setattr(os.path, "exists", lambda _: exists)

    run_called = [False]

    def mocked_run(_):
        run_called[0] = True

    monkeypatch.setattr(applycustomdnfconf, 'run', mocked_run)

    applycustomdnfconf.process()
    assert run_called[0] == should_move



from leapp.actors import Actor
from leapp.libraries.actor.checksaphana import perform_check
from leapp.models import SapHanaInfo
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckSapHana(Actor):
    """
    If SAP HANA has been detected, several checks are performed to ensure a successful upgrade.

    If the upgrade flavour is 'default' no checks are being executed.

    The following checks are executed:
    - If the major target release is 8, and this system is _NOT_ running on x86_64, the upgrade is inhibited.
    - If the major target release is 9, and this system is _NOT_ running on x86_64 or ppc64le,
      the upgrade is inhibited.
    - If SAP HANA 1 has been detected on the system the upgrade is inhibited since there is no supported upgrade path
      with installed SAP HANA 1.
    - If SAP HANA 2 has been detected, the upgrade will be inhibited if an unsupported version for the target release
      has been detected (<8.8, <9.2).
    - If the target release >=8.8 or >=9.2, the upgrade will be inhibited unless a user confirms to proceed
      for the currently installed SAP HANA 2.0 version and the chosen target release.
    - If SAP HANA is running the upgrade is inhibited.
    """

    name = 'check_sap_hana'
    consumes = (SapHanaInfo,)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        perform_check()



from leapp import reporting
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api
from leapp.models import StorageInfo

# man 5 xfs
REMOVED_XFS_OPTIONS = set([
    # removed from kernel in 4.0
    'nodelaylog',
    'delaylog',
    'ihashsize',
    'irixsgid',
    'osyncisdsync',
    'osyncisosync',
    # removed from kernel in 4.19
    'nobarrier',
    'barrier',
])


def _get_storage_data():
    storage = next(api.consume(StorageInfo), None)
    if not storage:
        raise StopActorExecutionError('The StorageInfo message is not available.')
    if not storage.fstab:
        raise StopActorExecutionError('Data from the /etc/fstab file is missing.')
    return storage


def process():
    storage = _get_storage_data()
    used_removed_options = set()
    for entry in storage.fstab:
        if entry.fs_vfstype == 'xfs':
            # NOTE: some opts could have a value, like ihashsize=4096 - we want
            # just the name of the option (that's why the double-split)
            options = set([opt.split('=')[0] for opt in entry.fs_mntops.split(',')])
            used_removed_options.update(options.intersection(REMOVED_XFS_OPTIONS))

    if not used_removed_options:
        return

    list_separator_fmt = '\n    - '
    reporting.create_report([
        reporting.Title('Deprecated XFS mount options present in FSTAB.'),
        reporting.Summary(
            'Some XFS mount options are not supported on RHEL 8 and prevent'
            ' system from booting correctly if any of the reported XFS options are used.'
            ' filesystem:{}{}.'.format(
                list_separator_fmt,
                list_separator_fmt.join(list(REMOVED_XFS_OPTIONS)))),
        reporting.Severity(reporting.Severity.HIGH),
        reporting.Groups([reporting.Groups.INHIBITOR]),
        reporting.Groups([reporting.Groups.FILESYSTEM]),
        reporting.RelatedResource('file', '/etc/fstab'),
        reporting.Remediation(hint=(
            'Drop the following mount options from the /etc/fstab file for any'
            ' XFS filesystem: {}.'.format(', '.join(used_removed_options)))),
    ])



import logging

from leapp.libraries.actor.rpmtransactionconfigtaskscollector import load_tasks, load_tasks_file
from leapp.libraries.stdlib import api
from leapp.models import DistributionSignedRPM, RPM

RH_PACKAGER = 'Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>'


def test_load_tasks(tmpdir, monkeypatch):

    def consume_signed_rpms_mocked(*models):
        installed = [
            RPM(name='c', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
                pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51')
            ]
        yield DistributionSignedRPM(items=installed)

    monkeypatch.setattr(api, "consume", consume_signed_rpms_mocked)

    tmpdir.join('to_install').write('a\n b\n  c \n\n\nc\na\nc\nb')
    tmpdir.join('to_keep').write('a\n b\n  c \n\n\nc\na\nc\nb')
    tmpdir.join('to_remove').write('a\n b\n  c \n\n\nc\na\nc\nb')
    m = load_tasks(tmpdir.strpath, logging)
    # c is not going to be in "to_install" as it is already installed
    assert set(m.to_install) == set(['a', 'b'])
    assert set(m.to_keep) == set(['a', 'b', 'c'])
    assert set(m.to_remove) == set(['a', 'b', 'c'])


def test_load_tasks_file(tmpdir):
    f = tmpdir.join('to_install')
    f.write('a\n b\n  c \n\n\nc\na\nc\nb')
    assert set(load_tasks_file(f.strpath, logging)) == set(['a', 'b', 'c'])
    f = tmpdir.join('to_keep')
    f.write(' ')
    assert set(load_tasks_file(f.strpath, logging)) == set([])



import functools

import pytest

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common import kernel as kernel_lib
from leapp.libraries.common.kernel import KernelType


@pytest.mark.parametrize(
    ('rhel_version', 'uname_r', 'expected_kernel_type'),
    (
        ('7.9', '3.10.0-1160.el7.x86_64', KernelType.ORDINARY),
        ('7.9', '3.10.0-1160.rt56.1131.el7.x86_64', KernelType.REALTIME),
        ('8.7', '4.18.0-425.3.1.el8.x86_64', KernelType.ORDINARY),
        ('8.7', '4.18.0-425.3.1.rt7.213.el8.x86_64', KernelType.REALTIME),
        ('9.2', '5.14.0-284.11.1.el9_2.x86_64', KernelType.ORDINARY),
        ('9.2', '5.14.0-284.11.1.rt14.296.el9_2.x86_64', KernelType.REALTIME),
        ('9.3', '5.14.0-354.el9.x86_64', KernelType.ORDINARY),
        ('9.3', '5.14.0-354.el9.x86_64+rt', KernelType.REALTIME),
    )
)
def test_determine_kernel_type_from_uname(rhel_version, uname_r, expected_kernel_type):
    kernel_type = kernel_lib.determine_kernel_type_from_uname(rhel_version, uname_r)
    assert kernel_type == expected_kernel_type


def test_get_uname_r_provided_by_kernel_pkg(monkeypatch):
    kernel_nevra = 'kernel-core-5.14.0-354.el9.x86_64'

    def run_mocked(cmd, *args, **kwargs):
        assert cmd == ['rpm', '-q', '--provides', kernel_nevra]
        output_lines = [
            'kmod(virtio_ring.ko)',
            'kernel(zlib_inflate_blob) = 0x65408378',
            'kernel-uname-r = 5.14.0-354.el9.x86_64'
        ]
        return {'stdout': output_lines}

    monkeypatch.setattr(kernel_lib, 'run', run_mocked)

    uname_r = kernel_lib.get_uname_r_provided_by_kernel_pkg(kernel_nevra)
    assert uname_r == '5.14.0-354.el9.x86_64'


@pytest.mark.parametrize('kernel_pkg_with_uname_installed', (True, False))
def test_get_kernel_pkg_info_for_uname_r(monkeypatch, kernel_pkg_with_uname_installed):
    uname_r = '5.14.0-354.el9.x86_64' if kernel_pkg_with_uname_installed else 'other-uname'

    def run_mocked(cmd, *args, **kwargs):
        assert cmd[0:3] == ['rpm', '-q', '--whatprovides']
        output_lines = [
            'kernel-core-5.14.0-354.el9.x86_64.rpm',
            'kernel-rt-core-5.14.0-354.el9.x86_64.rpm',
        ]
        return {'stdout': output_lines}

    def get_uname_provided_by_pkg_mocked(pkg_nevra):
        nevra_uname_table = {
            'kernel-core-5.14.0-354.el9.x86_64.rpm': '5.14.0-354.el9.x86_64',
            'kernel-rt-core-5.14.0-354.el9.x86_64.rpm': '5.14.0-354.el9.x86_64+rt'
        }
        return nevra_uname_table[pkg_nevra]  # Will raise if a different nevra is used than ones from run_mocked

    monkeypatch.setattr(kernel_lib, 'run', run_mocked)
    monkeypatch.setattr(kernel_lib, 'get_uname_r_provided_by_kernel_pkg', get_uname_provided_by_pkg_mocked)

    mk_pkg_info = functools.partial(kernel_lib.KernelPkgInfo, name='', version='', release='', arch='')
    monkeypatch.setattr(kernel_lib,
                        'get_kernel_pkg_info',
                        lambda dummy_nevra: mk_pkg_info(nevra=dummy_nevra))

    if kernel_pkg_with_uname_installed:
        pkg_info = kernel_lib.get_kernel_pkg_info_for_uname_r(uname_r)
        assert pkg_info == mk_pkg_info(nevra='kernel-core-5.14.0-354.el9.x86_64.rpm')
    else:
        with pytest.raises(StopActorExecutionError):
            pkg_info = kernel_lib.get_kernel_pkg_info_for_uname_r(uname_r)



import pytest
from six.moves.urllib.error import URLError

from leapp import reporting
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.actor.missinggpgkey import process
from leapp.libraries.common.gpg import get_pubkeys_from_rpms
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked, logger_mocked, produce_mocked
from leapp.libraries.stdlib import api
from leapp.models import (
    DNFWorkaround,
    GpgKey,
    Report,
    RepositoriesFacts,
    RepositoryData,
    RepositoryFile,
    RPM,
    TargetUserSpaceInfo,
    TMPTargetRepositoriesFacts,
    TrustedGpgKeys,
    UsedTargetRepositories,
    UsedTargetRepository
)
from leapp.utils.deprecation import suppress_deprecation

# Note, that this is not a real component test as described in the documentation,
# but basically unit test calling the "main" function process() to simulate the
# whole process as I was initially advised not to use these component tests.


def _get_test_gpgkeys_missing():
    """
    Return list of Trusted GPG keys without the epel9 key we look for
    """
    return [
        GpgKey(fingerprint='fd431d51', rpmdb=True),
        GpgKey(fingerprint='5a6340b3', rpmdb=True),
    ]


def _get_test_gpgkeys():
    """
    Return all the Trusted GPG keys for a test
    """
    return TrustedGpgKeys(items=[GpgKey(fingerprint='3228467c', rpmdb=True)] + _get_test_gpgkeys_missing())


def _get_test_targuserspaceinfo(path='/'):
    """
    Test TargetUserSpaceInfo which is needed to access the files in container root dir
    """
    return TargetUserSpaceInfo(
        path=path,
        scratch='',
        mounts='',
    )


def _get_test_usedtargetrepositories_list():
    """
    All target userspace directories
    """
    return [
        UsedTargetRepository(
            repoid='BaseOS',
        ),
        UsedTargetRepository(
            repoid='AppStream',
        ),
        UsedTargetRepository(
            repoid='MyAnotherRepo',
        ),
    ]


def _get_test_usedtargetrepositories():
    """
    The UsedTargetRepositories containing all repositories
    """
    return UsedTargetRepositories(
        repos=_get_test_usedtargetrepositories_list()
    )


def _get_test_target_repofile():
    """
    The valid RepositoryFile containing valid BaseOS and AppStream repositories
    """
    return RepositoryFile(
        file='/etc/yum.repos.d/target_rhel.repo',
        data=[
            RepositoryData(
                repoid='BaseOS',
                name="RHEL BaseOS repository",
                baseurl="/whatever/",
                enabled=True,
                additional_fields='{"gpgkey":"file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release"}'
            ),
            RepositoryData(
                repoid='AppStream',
                name="RHEL AppStream repository",
                baseurl="/whatever/",
                enabled=True,
                additional_fields='{"gpgkey":"file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release"}'
            ),
        ],
    )


def _get_test_target_repofile_additional():
    """
    The custom target repofile containing "problematic" repositories
    """
    return RepositoryFile(
        file='/etc/yum.repos.d/my_target_rhel.repo',
        data=[
            RepositoryData(
                repoid='MyRepo',
                name="My repository",
                baseurl="/whatever/",
                enabled=False,
            ),
            RepositoryData(
                repoid='MyAnotherRepo',
                name="My another repository",
                baseurl="/whatever/",
                enabled=True,
                additional_fields='{"gpgkey":"file:///etc/pki/rpm-gpg/RPM-GPG-KEY-my-release"}'
            ),
        ],
    )


@suppress_deprecation(TMPTargetRepositoriesFacts)
def _get_test_tmptargetrepositoriesfacts():
    """
    All target repositories facts
    """
    return TMPTargetRepositoriesFacts(
        repositories=[
            _get_test_target_repofile(),
            _get_test_target_repofile_additional(),
        ],
    )


def test_perform_nogpgcheck(monkeypatch):
    """
    Executes the "main" function with the --nogpgcheck commandline switch

    This test should skip any checks and just log a message that no checks were executed
    """
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        envars={'LEAPP_NOGPGCHECK': '1'},
        msgs=[
            _get_test_gpgkeys(),
            _get_test_usedtargetrepositories(),
            _get_test_tmptargetrepositoriesfacts(),
        ],
    ))
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    process()

    assert api.produce.called == 0
    assert len(api.current_logger.warnmsg) == 1
    assert '--nogpgcheck option is used' in api.current_logger.warnmsg[0]


@pytest.mark.parametrize('msgs', [
    [],
    [_get_test_gpgkeys],
    [_get_test_usedtargetrepositories],
    [_get_test_tmptargetrepositoriesfacts],
    # These are just incomplete lists of required facts
    [_get_test_gpgkeys(), _get_test_usedtargetrepositories()],
    [_get_test_usedtargetrepositories(), _get_test_tmptargetrepositoriesfacts()],
    [_get_test_gpgkeys(), _get_test_tmptargetrepositoriesfacts()],
])
def test_perform_missing_facts(monkeypatch, msgs):
    """
    Executes the "main" function with missing required facts

    The missing facts (either RPM information, Target Repositories or their facts) cause
    the StopActorExecutionError excepction. But this should be rare as the required facts
    are clearly defined in the actor interface.
    """
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=msgs))
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    # TODO: the gpg call should be mocked

    with pytest.raises(StopActorExecutionError):
        process()
    # nothing produced
    assert api.produce.called == 0
    # not skipped by --nogpgcheck
    assert not api.current_logger.warnmsg


@suppress_deprecation(TMPTargetRepositoriesFacts)
def _get_test_tmptargetrepositoriesfacts_partial():
    return [
        _get_test_gpgkeys(),
        _get_test_usedtargetrepositories(),
        TMPTargetRepositoriesFacts(
            repositories=[
                _get_test_target_repofile(),
                # missing MyAnotherRepo
            ]
        )
    ]


def _gpg_show_keys_mocked(key_path):
    """
    Get faked output from gpg reading keys.

    This is needed to get away from dependency on the filesystem
    """
    if key_path == '/etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release':
        return {
            'stdout': [
                'pub:-:4096:1:199E2F91FD431D51:1256212795:::-:::scSC::::::23::0:',
                'fpr:::::::::567E347AD0044ADE55BA8A5F199E2F91FD431D51:',
                ('uid:-::::1256212795::DC1CAEC7997B3575101BB0FCAAC6191792660D8F::'
                 'Red Hat, Inc. (release key 2) <security@redhat.com>::::::::::0:'),
                'pub:-:4096:1:5054E4A45A6340B3:1646863006:::-:::scSC::::::23::0:',
                'fpr:::::::::7E4624258C406535D56D6F135054E4A45A6340B3:',
                ('uid:-::::1646863006::DA7F68E3872D6E7BDCE05225E7EB5F3ACDD9699F::'
                 'Red Hat, Inc. (auxiliary key 3) <security@redhat.com>::::::::::0:'),
            ],
            'stderr': (),
            'exit_code': 0,
        }
    if key_path == '/etc/pki/rpm-gpg/RPM-GPG-KEY-my-release':  # actually epel9 key
        return {
            'stdout': [
                'pub:-:4096:1:8A3872BF3228467C:1631033579:::-:::escESC::::::23::0:',
                'fpr:::::::::FF8AD1344597106ECE813B918A3872BF3228467C:',
                ('uid:-::::1631033579::3EED52B2BDE50880047DB883C87B0FCAE458D111::'
                 'Fedora (epel9) <epel@fedoraproject.org>::::::::::0:'),
            ],
            'stderr': (),
            'exit_code': 0,
        }

    return {
        'stdout': [
            'pub:-:4096:1:F55AD3FB5323552A:1628617948:::-:::escESC::::::23::0:',
            'fpr:::::::::ACB5EE4E831C74BB7C168D27F55AD3FB5323552A:',
            ('uid:-::::1628617948::4830BB019772421B89ABD0BBE245B89C73BF053F::'
             'Fedora (37) <fedora-37-primary@fedoraproject.org>::::::::::0:'),
        ],
        'stderr': (),
        'exit_code': 0,
    }


def _get_pubkeys_mocked(installed_rpms):
    """
    This skips getting fps from files in container for simplification
    """
    return get_pubkeys_from_rpms(installed_rpms)


def test_perform_missing_some_repo_facts(monkeypatch):
    """
    Executes the "main" function with missing repositories facts

    This is misalignment in the provided facts UsedTargetRepositories and TMPTargetRepositoriesFacts,
    where we miss some metadata that are required by the first message.
    """
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=_get_test_tmptargetrepositoriesfacts_partial())
    )
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr('leapp.libraries.common.gpg._gpg_show_keys', _gpg_show_keys_mocked)

    with pytest.raises(StopActorExecutionError):
        process()
    assert api.produce.called == 0
    assert reporting.create_report.called == 0


@suppress_deprecation(TMPTargetRepositoriesFacts)
def _get_test_tmptargetrepositoriesfacts_https_unused():
    return [
        _get_test_targuserspaceinfo(),
        _get_test_gpgkeys(),
        _get_test_usedtargetrepositories(),
        TMPTargetRepositoriesFacts(
            repositories=[
                _get_test_target_repofile(),
                _get_test_target_repofile_additional(),
                RepositoryFile(
                    file='/etc/yum.repos.d/internet.repo',
                    data=[
                        RepositoryData(
                            repoid='ExternalRepo',
                            name="External repository",
                            baseurl="/whatever/path",
                            enabled=True,
                            additional_fields='{"gpgkey":"https://example.com/rpm-gpg/key.gpg"}',
                        ),
                    ],
                )
            ],
        ),
    ]


def test_perform_https_gpgkey_unused(monkeypatch):
    """
    Executes the "main" function with repositories providing keys over internet

    The external repository is not listed in UsedTargetRepositories so the repository
    is not checked and we should not get any error here.
    """
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=_get_test_tmptargetrepositoriesfacts_https_unused()
    ))
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr('leapp.libraries.common.gpg._gpg_show_keys', _gpg_show_keys_mocked)

    process()
    assert not api.current_logger.warnmsg
    assert api.produce.called == 1
    assert isinstance(api.produce.model_instances[0], DNFWorkaround)
    assert reporting.create_report.called == 0


@suppress_deprecation(TMPTargetRepositoriesFacts)
def get_test_tmptargetrepositoriesfacts_https():
    return (
        _get_test_targuserspaceinfo(),
        _get_test_gpgkeys(),
        UsedTargetRepositories(
            repos=_get_test_usedtargetrepositories_list() + [
                UsedTargetRepository(
                    repoid='ExternalRepo',
                ),
            ]
        ),
        TMPTargetRepositoriesFacts(
            repositories=[
                _get_test_target_repofile(),
                _get_test_target_repofile_additional(),
                RepositoryFile(
                    file='/etc/yum.repos.d/internet.repo',
                    data=[
                        RepositoryData(
                            repoid='ExternalRepo',
                            name="External repository",
                            baseurl="/whatever/path",
                            enabled=True,
                            additional_fields='{"gpgkey":"https://example.com/rpm-gpg/key.gpg"}',
                        ),
                    ],
                )
            ],
        ),
    )


@suppress_deprecation(TMPTargetRepositoriesFacts)
def get_test_tmptargetrepositoriesfacts_ftp():
    return (
        _get_test_targuserspaceinfo(),
        _get_test_gpgkeys(),
        UsedTargetRepositories(
            repos=_get_test_usedtargetrepositories_list() + [
                UsedTargetRepository(
                    repoid='ExternalRepo',
                ),
            ]
        ),
        TMPTargetRepositoriesFacts(
            repositories=[
                _get_test_target_repofile(),
                _get_test_target_repofile_additional(),
                RepositoryFile(
                    file='/etc/yum.repos.d/internet.repo',
                    data=[
                        RepositoryData(
                            repoid='ExternalRepo',
                            name="External repository",
                            baseurl="/whatever/path",
                            enabled=True,
                            additional_fields='{"gpgkey":"ftp://example.com/rpm-gpg/key.gpg"}',
                        ),
                    ],
                )
            ],
        ),
    )


def _urlretrive_mocked(url, filename=None, reporthook=None, data=None):
    return filename


def test_perform_https_gpgkey(monkeypatch):
    """
    Executes the "main" function with repositories providing keys over internet

    This produces an report.
    """
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=get_test_tmptargetrepositoriesfacts_https())
    )
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr('leapp.libraries.common.gpg._gpg_show_keys', _gpg_show_keys_mocked)
    monkeypatch.setattr('six.moves.urllib.request.urlretrieve', _urlretrive_mocked)

    process()
    assert api.produce.called == 1
    assert isinstance(api.produce.model_instances[0], DNFWorkaround)
    assert reporting.create_report.called == 1
    assert "Detected unknown GPG keys for target system repositories" in reporting.create_report.reports[0]['title']
    assert "https://example.com/rpm-gpg/key.gpg" in reporting.create_report.reports[0]['summary']


def _urlretrive_mocked_urlerror(url, filename=None, reporthook=None, data=None):
    raise URLError('error')


def test_perform_https_gpgkey_urlerror(monkeypatch):
    """
    Executes the "main" function with repositories providing keys over internet

    This results in warning message printed. Other than that, no report is still produced.
    """
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=get_test_tmptargetrepositoriesfacts_https())
    )
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr('leapp.libraries.common.gpg._gpg_show_keys', _gpg_show_keys_mocked)
    monkeypatch.setattr('six.moves.urllib.request.urlretrieve', _urlretrive_mocked_urlerror)

    process()
    assert len(api.current_logger.warnmsg) == 1
    assert 'Failed to download the gpgkey https://example.com/rpm-gpg/key.gpg:' in api.current_logger.warnmsg[0]
    assert api.produce.called == 1
    assert isinstance(api.produce.model_instances[0], DNFWorkaround)
    assert reporting.create_report.called == 1
    assert "Failed to download GPG key for target repository" in reporting.create_report.reports[0]['title']
    assert "https://example.com/rpm-gpg/key.gpg" in reporting.create_report.reports[0]['summary']


def test_perform_ftp_gpgkey(monkeypatch):
    """
    Executes the "main" function with repositories providing keys over internet

    This results in error message printed. Other than that, no report is still produced.
    """
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=get_test_tmptargetrepositoriesfacts_ftp())
    )
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr('leapp.libraries.common.gpg._gpg_show_keys', _gpg_show_keys_mocked)

    process()
    assert len(api.current_logger.errmsg) == 1
    assert 'Skipping unknown protocol for gpgkey ftp://example.com/rpm-gpg/key.gpg' in api.current_logger.errmsg[0]
    assert api.produce.called == 1
    assert isinstance(api.produce.model_instances[0], DNFWorkaround)
    assert reporting.create_report.called == 1
    assert 'GPG keys provided using unknown protocol' in reporting.create_report.reports[0]['title']
    assert 'ftp://example.com/rpm-gpg/key.gpg' in reporting.create_report.reports[0]['summary']


@suppress_deprecation(TMPTargetRepositoriesFacts)
def get_test_data_missing_key():
    return [
        _get_test_targuserspaceinfo(),
        TrustedGpgKeys(items=_get_test_gpgkeys_missing()),
        _get_test_usedtargetrepositories(),
        _get_test_tmptargetrepositoriesfacts(),
    ]


def test_perform_report(monkeypatch):
    """
    Executes the "main" function with missing keys

    This should result in report outlining what key mentioned in target repositories is missing.
    """
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=get_test_data_missing_key())
    )
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr('leapp.libraries.common.gpg._gpg_show_keys', _gpg_show_keys_mocked)

    process()
    assert not api.current_logger.warnmsg
    assert api.produce.called == 1
    assert isinstance(api.produce.model_instances[0], DNFWorkaround)
    assert reporting.create_report.called == 1
    assert "Detected unknown GPG keys for target system repositories" in reporting.create_report.reports[0]['title']
    assert "/etc/pki/rpm-gpg/RPM-GPG-KEY-my-release" in reporting.create_report.reports[0]['summary']


@suppress_deprecation(TMPTargetRepositoriesFacts)
def get_test_data_no_gpg_data():
    return [
        _get_test_targuserspaceinfo(),
        _get_test_gpgkeys(),
        _get_test_usedtargetrepositories(),
        _get_test_tmptargetrepositoriesfacts(),
    ]


def _gpg_show_keys_mocked_my_empty(key_path):
    """
    Get faked output from gpg reading keys.

    This is needed to get away from dependency on the filesystem. This time, the key
    /etc/pki/rpm-gpg/RPM-GPG-KEY-my-release does not return any GPG data
    """
    if key_path == '/etc/pki/rpm-gpg/RPM-GPG-KEY-my-release':
        return {
            'stdout': (),
            'stderr': ('gpg: no valid OpenPGP data found.\n'),
            'exit_code': 2,
        }
    return _gpg_show_keys_mocked(key_path)


def test_perform_invalid_key(monkeypatch):
    """
    Executes the "main" function with a gpgkey not containing any GPG data

    This should result in report outlining what key does not contain any valid data.
    """
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=get_test_data_no_gpg_data())
    )
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr('leapp.libraries.common.gpg._gpg_show_keys', _gpg_show_keys_mocked_my_empty)

    process()
    assert len(api.current_logger.warnmsg) == 2, api.current_logger.warnmsg
    assert 'Cannot get any gpg key from the file' in api.current_logger.warnmsg[1]
    assert api.produce.called == 1
    assert isinstance(api.produce.model_instances[0], DNFWorkaround)
    assert reporting.create_report.called == 1
    assert 'Failed to read GPG keys from provided key files' in reporting.create_report.reports[0]['title']
    assert 'file:///etc/pki/rpm-gpg/RPM-GPG-KEY-my-release' in reporting.create_report.reports[0]['summary']


@suppress_deprecation(TMPTargetRepositoriesFacts)
def get_test_data_gpgcheck_without_gpgkey():
    return [
        _get_test_targuserspaceinfo(),
        _get_test_gpgkeys(),
        UsedTargetRepositories(
            repos=_get_test_usedtargetrepositories_list() + [
                UsedTargetRepository(
                    repoid='InvalidRepo',
                ),
            ]
        ),
        TMPTargetRepositoriesFacts(
            repositories=[
                _get_test_target_repofile(),
                _get_test_target_repofile_additional(),
                RepositoryFile(
                    file='/etc/yum.repos.d/invalid.repo',
                    data=[
                        RepositoryData(
                            repoid='InvalidRepo',
                            name="Invalid repository",
                            baseurl="/whatever/path",
                            enabled=True,
                            additional_fields='{"gpgcheck":"1"}',  # this should be default
                        ),
                    ],
                )
            ],
        ),
    ]


def test_perform_gpgcheck_without_gpgkey(monkeypatch):
    """
    Executes the "main" function with a repository containing a gpgcheck=1 without any gpgkey=

    This should result in report outlining that this configuration is not supported
    """
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(
        msgs=get_test_data_gpgcheck_without_gpgkey())
    )
    monkeypatch.setattr(api, 'produce', produce_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr('leapp.libraries.common.gpg._gpg_show_keys', _gpg_show_keys_mocked)

    process()
    assert len(api.current_logger.warnmsg) == 1
    assert ('The gpgcheck for the InvalidRepo repository is enabled but gpgkey is not specified.'
            ' Cannot be checked.') in api.current_logger.warnmsg[0]
    assert api.produce.called == 1
    assert isinstance(api.produce.model_instances[0], DNFWorkaround)
    assert reporting.create_report.called == 1
    assert 'Inconsistent repository without GPG key' in reporting.create_report.reports[0]['title']
    assert 'InvalidRepo' in reporting.create_report.reports[0]['summary']



from leapp.actors import Actor
from leapp.models import NetworkManagerConfig
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag

snippet_path = '/usr/lib/NetworkManager/conf.d/10-dhcp-dhclient.conf'
snippet_data = ("# Generated by leapp when upgrading from RHEL7 to RHEL8\n"
                "[main]\n"
                "dhcp=dhclient\n")


class NetworkManagerUpdateConfig(Actor):
    """
    Updates NetworkManager configuration for Red Hat Enterprise Linux 8.

    On Red Hat Enterprise Linux 7 NetworkManager uses the "dhclient" DHCP backend by default, while
    the default is "internal" on Red Hat Enterprise Linux 8. We want to keep "dhclient" enabled on
    upgrade, unless the user explicitly chose another backend in the configuration. To do so, we
    drop a configuration snippet in /usr/lib.
    """

    name = 'network_manager_update_config'
    consumes = (NetworkManagerConfig,)
    produces = ()
    tags = (ApplicationsPhaseTag, IPUWorkflowTag)

    def process(self):
        for nm_config in self.consume(NetworkManagerConfig):
            self.log.info('Consuming dhcp={}'.format(nm_config.dhcp))
            if nm_config.dhcp == '':
                try:
                    with open(snippet_path, 'w') as f:
                        f.write(snippet_data)
                        self.log.info('Written the following to {}:\n{}\n'.format(snippet_path, snippet_data))
                except IOError as e:
                    self.log.warning('Write error: {}'.format(e))
            break



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class UdevAdmInfoData(Model):
    topic = SystemInfoTopic

    db = fields.String()
    """Database export obtained by executing 'udevadm info -e'."""



from leapp.models import Interface, KernelCmdlineArg, PCIAddress, PersistentNetNamesFacts
from leapp.reporting import Report
from leapp.snactor.fixture import current_actor_context
from leapp.utils.report import is_inhibitor


def test_actor_single_eth0(current_actor_context):
    pci = PCIAddress(domain="0000", bus="3e", function="00", device="PCI bridge")
    interface = [Interface(name="eth0", mac="52:54:00:0b:4a:6d", vendor="redhat",
                           driver="pcieport", pci_info=pci,
                           devpath="/devices/platform/usb/cdc-wdm0")]
    current_actor_context.feed(PersistentNetNamesFacts(interfaces=interface))
    current_actor_context.run()
    assert not current_actor_context.consume(Report)


def test_actor_more_ethX(current_actor_context):
    pci1 = PCIAddress(domain="0000", bus="3e", function="00", device="PCI bridge")
    pci2 = PCIAddress(domain="0000", bus="3d", function="00", device="Serial controller")
    interface = [Interface(name="eth0", mac="52:54:00:0b:4a:6d", vendor="redhat",
                           driver="pcieport", pci_info=pci1,
                           devpath="/devices/platform/usb/cdc-wdm0"),
                 Interface(name="eth1", mac="52:54:00:0b:4a:6a", vendor="redhat",
                           driver="serial", pci_info=pci2,
                           devpath="/devices/hidraw/hidraw0")]
    current_actor_context.feed(PersistentNetNamesFacts(interfaces=interface))
    current_actor_context.run()
    assert current_actor_context.consume(Report)
    assert is_inhibitor(current_actor_context.consume(Report)[0].report)


def test_actor_single_int_not_ethX(current_actor_context):
    pci = PCIAddress(domain="0000", bus="3e", function="00", device="PCI bridge")
    interface = [Interface(name="tap0", mac="52:54:00:0b:4a:60", vendor="redhat",
                           driver="pcieport", pci_info=pci,
                           devpath="/devices/platform/usb/cdc-wdm0")]
    current_actor_context.feed(PersistentNetNamesFacts(interfaces=interface))
    current_actor_context.run()
    assert not current_actor_context.consume(Report)


def test_actor_ethX_and_not_ethX(current_actor_context):
    pci1 = PCIAddress(domain="0000", bus="3e", function="00", device="PCI bridge")
    pci2 = PCIAddress(domain="0000", bus="3d", function="00", device="Serial controller")
    interface = [Interface(name="virbr0", mac="52:54:00:0b:4a:6d", vendor="redhat",
                           driver="pcieport", pci_info=pci1,
                           devpath="/devices/platform/usb/cdc-wdm0"),
                 Interface(name="eth0", mac="52:54:00:0b:4a:6a", vendor="redhat",
                           driver="serial", pci_info=pci2,
                           devpath="/devices/hidraw/hidraw0")]
    current_actor_context.feed(PersistentNetNamesFacts(interfaces=interface))
    current_actor_context.run()
    assert current_actor_context.consume(Report)
    assert is_inhibitor(current_actor_context.consume(Report)[0].report)



from leapp.actors import Actor
from leapp.libraries.actor import networkmanagerconnectionscanner
from leapp.models import NetworkManagerConnection
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class NetworkManagerConnectionScanner(Actor):
    """
    Scan NetworkManager connection keyfiles
    """

    name = "network_manager_connection_scanner"
    consumes = ()
    produces = (NetworkManagerConnection,)
    tags = (IPUWorkflowTag, FactsPhaseTag,)

    def process(self):
        networkmanagerconnectionscanner.process()



from leapp.actors import Actor
from leapp.libraries.actor import storagescanner
from leapp.models import StorageInfo
from leapp.reporting import Report
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class StorageScanner(Actor):
    """
    Provides data about storage settings.

    After collecting data from tools like mount, lsblk, pvs, vgs and lvdisplay, and relevant files
    under /proc/partitions and /etc/fstab, a message with relevant data will be produced.
    """

    name = 'storage_scanner'
    consumes = ()
    produces = (Report, StorageInfo)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        self.produce(storagescanner.get_storage_info())



import errno
import os

from leapp.libraries.actor import config_parser
from leapp.libraries.common import vsftpdutils as utils
from leapp.libraries.stdlib import api
from leapp.models import VsftpdConfig, VsftpdFacts


def _parse_config(path, content):
    try:
        parser = config_parser.VsftpdConfigParser(content)
        return parser.parsed_config
    except config_parser.ParsingError:
        api.current_logger().info('File %s does not look like vsftpd configuration, skipping.'
                                  % path)
        return None


def _get_parsed_configs(read_func=utils.read_file, listdir=os.listdir):
    res = []
    try:
        for fname in listdir(utils.VSFTPD_CONFIG_DIR):
            path = os.path.join(utils.VSFTPD_CONFIG_DIR, fname)
            if not path.endswith('.conf'):
                continue
            content = utils.get_config_contents(path, read_func=read_func)
            if content is None:
                continue
            parsed = _parse_config(path, content)
            if parsed is not None:
                res.append((path, parsed))
    except OSError as e:
        if e.errno != errno.ENOENT:
            api.current_logger().warning('Failed to read vsftpd configuration directory: %s'
                                         % e)
    return res


def get_vsftpd_facts(read_func=utils.read_file, listdir=os.listdir):
    config_hash = utils.get_default_config_hash(read_func=read_func)
    configs = _get_parsed_configs(read_func=read_func, listdir=listdir)
    res_configs = []
    for path, config in configs:
        res_configs.append(VsftpdConfig(path=path,
                                        strict_ssl_read_eof=config.get(utils.STRICT_SSL_READ_EOF),
                                        tcp_wrappers=config.get(utils.TCP_WRAPPERS)))
    return VsftpdFacts(default_config_hash=config_hash, configs=res_configs)


def is_processable(installed_rpm_facts):
    for pkg in installed_rpm_facts.items:
        if pkg.name == 'vsftpd':
            return True
    return False



import os
from collections import defaultdict

from leapp import reporting
from leapp.libraries.stdlib import api
from leapp.models import TargetInitramfsTasks, UpgradeInitramfsTasks

DRACUT_MOD_DIR = '/usr/lib/dracut/modules.d/'
SUMMARY_FMT = (
    'The requested {kind} modules for the initramfs are in conflict.'
    ' At least one {kind} module is specified to be installed from'
    ' multiple paths. The list of conflicting {kind} module names'
    ' with paths is listed below: {conflicts}'
)


def _printable_modules(conflicts):
    list_separator_fmt = '\n    - '
    for name, paths in conflicts.items():
        paths = sorted([str(i) for i in paths])
        output = ['{}{}: {}'.format(list_separator_fmt, name, paths)]
    return ''.join(output)


def _treat_path_dracut(dmodule):
    """
    In case the path is not set, set the expected path of the dracut module.
    """

    if not dmodule.module_path:
        return os.path.join(DRACUT_MOD_DIR, dmodule.name)
    return dmodule.module_path


def _treat_path_kernel(kmodule):
    """
    In case the path of a kernel module is not set, indicate that the module is
    taken from the current system.
    """

    if not kmodule.module_path:
        return kmodule.name + ' (system)'
    return kmodule.module_path


def _detect_modules_conflicts(msgtype, kind):
    """
    Return dict of modules with conflicting tasks

    In this case when a module should be applied but different sources are
    specified. E.g.:
       include modules X where,
         msg A)  X
         msg B)  X from custom path
    """

    modules_map = {
        'dracut': {
            'msgattr': 'include_dracut_modules',
            'treat_path_fn': _treat_path_dracut,
        },
        'kernel': {
            'msgattr': 'include_kernel_modules',
            'treat_path_fn': _treat_path_kernel
        },
    }

    modules = defaultdict(set)
    for msg in api.consume(msgtype):
        for module in getattr(msg, modules_map[kind]['msgattr']):
            treat_path_fn = modules_map[kind]['treat_path_fn']
            modules[module.name].add(treat_path_fn(module))
    return {key: val for key, val in modules.items() if len(val) > 1}


def report_conflicts(msgname, kind, msgtype):
    conflicts = _detect_modules_conflicts(msgtype, kind)
    if not conflicts:
        return
    report = [
        reporting.Title('Conflicting requirements of {kind} modules for the {msgname} initramfs'.format(
            kind=kind, msgname=msgname)),
        reporting.Summary(SUMMARY_FMT.format(kind=kind, conflicts=_printable_modules(conflicts))),
        reporting.Severity(reporting.Severity.HIGH),
        reporting.Groups([reporting.Groups.SANITY, reporting.Groups.INHIBITOR]),
    ]
    reporting.create_report(report)


def process():
    report_conflicts('upgrade', 'kernel', UpgradeInitramfsTasks)
    report_conflicts('upgrade', 'dracut', UpgradeInitramfsTasks)
    report_conflicts('target', 'dracut', TargetInitramfsTasks)



from leapp.models import BrlttyMigrationDecision, DistributionSignedRPM, RPM
from leapp.reporting import Report

RH_PACKAGER = 'Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>'

with_brltty = [
    RPM(name='grep', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
        pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51'),
    RPM(name='brltty', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
        pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51')]

without_brltty = [
    RPM(name='grep', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
        pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51'),
    RPM(name='sed', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
        pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51')]


def create_modulesfacts(installed_rpm):
    return DistributionSignedRPM(items=installed_rpm)


def test_actor_without_brltty_package(current_actor_context):
    current_actor_context.feed(create_modulesfacts(installed_rpm=without_brltty))
    current_actor_context.run()
    assert not current_actor_context.consume(Report)


def test_actor_with_brltty_package(current_actor_context):
    current_actor_context.feed(create_modulesfacts(installed_rpm=with_brltty))
    current_actor_context.run()
    assert current_actor_context.consume(Report)



from leapp.actors import Actor
from leapp.libraries.actor import checksystemarch
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckSystemArch(Actor):
    """
    Check if system is running at a supported architecture. If no, inhibit the upgrade process.

    Base on collected system facts, verify if current architecture is supported, otherwise produces
    a message to inhibit upgrade process
    """

    name = 'check_system_arch'
    consumes = ()
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        checksystemarch.check_architecture()



from leapp.libraries import stdlib
from leapp.libraries.common.config import architecture
from leapp.libraries.stdlib import api, config
from leapp.models import InstalledTargetKernelInfo


def update_default_kernel(kernel_info):
    try:
        stdlib.run(['grubby', '--info', kernel_info.kernel_img_path])
    except stdlib.CalledProcessError:
        api.current_logger().error('Expected kernel %s to be installed at the boot loader but cannot be found.',
                                   kernel_info.kernel_img_path)
    except OSError:
        api.current_logger().error('Could not check for kernel existence in boot loader. Is grubby installed?')
    else:
        try:
            stdlib.run(['grubby', '--set-default', kernel_info.kernel_img_path])
            if architecture.matches_architecture(architecture.ARCH_S390X):
                # on s390x we need to call zipl explicitly because of issue in grubby,
                # otherwise the new boot entry will not be set as default
                # See https://bugzilla.redhat.com/show_bug.cgi?id=1764306
                stdlib.run(['/usr/sbin/zipl'])
        except (OSError, stdlib.CalledProcessError):
            api.current_logger().error('Failed to set default kernel to: %s',
                                       kernel_info.kernel_img_path, exc_info=True)


def process():
    is_system_s390x = architecture.matches_architecture(architecture.ARCH_S390X)
    if config.is_debug and not is_system_s390x:  # pylint: disable=using-constant-test
        try:
            # the following command prints output of grubenv for debugging purposes and is repeated after setting
            # default kernel so we can be sure we have the right saved entry
            #
            # saved_entry=63...
            # kernelopts=root=/dev/mapper...
            #
            #
            # boot_success and boot_indeterminate parameters are added later by one-shot systemd service
            stdlib.run(['grub2-editenv', 'list'])
        except stdlib.CalledProcessError:
            api.current_logger().error('Failed to execute "grub2-editenv list" command')

    kernel_info = next(api.consume(InstalledTargetKernelInfo), None)
    if not kernel_info:
        api.current_logger().warning(('Skipped - Forcing checking and setting default boot entry to target kernel'
                                      ' version due to missing message'))
        return

    if not kernel_info.kernel_img_path:  # Should be always set
        api.current_logger().warning(('Skipping forcing of default boot entry - target kernel info '
                                      'does not contain a kernel image path.'))
        return

    try:
        current_default_kernel = stdlib.run(['grubby', '--default-kernel'])['stdout'].strip()
    except (OSError, stdlib.CalledProcessError):
        api.current_logger().warning('Failed to query grubby for default kernel', exc_info=True)
        return

    for type_ in ('index', 'title'):
        try:
            stdlib.run(['grubby', '--default-{}'.format(type_)])
        except (OSError, stdlib.CalledProcessError):
            api.current_logger().warning('Failed to query grubby for default {}'.format(type_), exc_info=True)
            return

    if current_default_kernel != kernel_info.kernel_img_path:
        api.current_logger().warning(('Current default boot entry not target kernel version: Current default: %s.'
                                      'Forcing default kernel to %s'),
                                     current_default_kernel, kernel_info.kernel_img_path)
        update_default_kernel(kernel_info)
    if config.is_debug and not is_system_s390x:  # pylint: disable=using-constant-test
        try:
            stdlib.run(['grub2-editenv', 'list'])
        except stdlib.CalledProcessError:
            api.current_logger().error('Failed to execute "grub2-editenv list" command')



import errno
import textwrap

import pytest
import six

from leapp.libraries.actor import networkmanagerconnectionscanner as nmconnscanner
from leapp.libraries.common.testutils import make_OSError, produce_mocked
from leapp.libraries.stdlib import api
from leapp.models import NetworkManagerConnection

_builtins_open = "builtins.open" if six.PY3 else "__builtin__.open"


def _listdir_nm_conn(path):
    if path == nmconnscanner.NM_CONN_DIR:
        return ["conn1.nmconnection"]
    raise make_OSError(errno.ENOENT)


def _listdir_nm_conn2(path):
    if path == nmconnscanner.NM_CONN_DIR:
        return ["conn1.nmconnection", "conn2.nmconnection"]
    raise make_OSError(errno.ENOENT)


def _load_from_file(keyfile, filename, flags):
    if filename.endswith(".nmconnection"):
        return keyfile.load_from_data(textwrap.dedent("""
            [connection]
            type=wifi
            id=conn1
            uuid=a1bc695d-c548-40e8-9c7f-205a6587135d

            [wifi]
            mode=infrastructure
            ssid=wifi

            [wifi-security]
            auth-alg=open
            key-mgmt=none
            wep-key-type=1
            wep-key0=abcde
        """), nmconnscanner.GLib.MAXSIZE, flags)
    raise make_OSError(errno.ENOENT)


@pytest.mark.skipif(not nmconnscanner.libnm_available, reason="NetworkManager g-ir not installed")
def test_no_conf(monkeypatch):
    """
    No report if there are no keyfiles
    """

    monkeypatch.setattr(nmconnscanner.os, "listdir", lambda _: ())
    monkeypatch.setattr(api, "produce", produce_mocked())
    nmconnscanner.process()
    assert not api.produce.called


@pytest.mark.skipif(not nmconnscanner.libnm_available, reason="NetworkManager g-ir not installed")
def test_nm_conn(monkeypatch):
    """
    Check a basic keyfile
    """

    monkeypatch.setattr(nmconnscanner.os, "listdir", _listdir_nm_conn)
    monkeypatch.setattr(api, "produce", produce_mocked())
    monkeypatch.setattr(nmconnscanner.GLib.KeyFile, "load_from_file", _load_from_file)
    nmconnscanner.process()

    assert api.produce.called == 1
    assert len(api.produce.model_instances) == 1
    nm_conn = api.produce.model_instances[0]
    assert isinstance(nm_conn, NetworkManagerConnection)
    assert nm_conn.filename == "/etc/NetworkManager/system-connections/conn1.nmconnection"
    assert len(nm_conn.settings) == 3
    assert nm_conn.settings[0].name == "connection"
    assert len(nm_conn.settings[0].properties) == 4
    assert nm_conn.settings[0].properties[0].name == "id"
    assert nm_conn.settings[0].properties[0].value == "conn1"
    assert nm_conn.settings[2].name == "wifi-security"

    # It's important that wek-key0 is gone
    assert len(nm_conn.settings[2].properties) == 3
    assert nm_conn.settings[2].properties[0].name == "auth-alg"
    assert nm_conn.settings[2].properties[0].value == "open"
    assert nm_conn.settings[2].properties[1].name != "wep-key0"
    assert nm_conn.settings[2].properties[2].name != "wep-key0"


@pytest.mark.skipif(not nmconnscanner.libnm_available, reason="NetworkManager g-ir not installed")
def test_nm_conn2(monkeypatch):
    """
    Check a pair of keyfiles
    """

    monkeypatch.setattr(nmconnscanner.os, "listdir", _listdir_nm_conn2)
    monkeypatch.setattr(api, "produce", produce_mocked())
    monkeypatch.setattr(nmconnscanner.GLib.KeyFile, "load_from_file", _load_from_file)
    nmconnscanner.process()

    assert api.produce.called == 2
    assert len(api.produce.model_instances) == 2
    assert api.produce.model_instances[0].filename.endswith("/conn1.nmconnection")
    assert api.produce.model_instances[1].filename.endswith("/conn2.nmconnection")



from leapp.libraries.actor import checkcustommodifications
from leapp.models import CustomModifications, Report


def test_report_any_modifications(current_actor_context):
    discovered_msgs = [CustomModifications(filename='some/changed/leapp/actor/file',
                                           type='modified',
                                           actor_name='an_actor',
                                           component='repository'),
                       CustomModifications(filename='some/new/actor/in/leapp/dir',
                                           type='custom',
                                           actor_name='a_new_actor',
                                           component='repository'),
                       CustomModifications(filename='some/new/actor/in/leapp/dir',
                                           type='modified',
                                           actor_name='a_new_actor',
                                           component='configuration'),
                       CustomModifications(filename='some/changed/file/in/framework',
                                           type='modified',
                                           actor_name='',
                                           component='framework')]
    for msg in discovered_msgs:
        current_actor_context.feed(msg)
    current_actor_context.run()
    reports = current_actor_context.consume(Report)
    assert len(reports) == 3
    assert (reports[0].report['title'] ==
            'Detected custom leapp actors or files.')
    assert 'some/new/actor/in/leapp/dir (Actor: a_new_actor)' in reports[0].report['summary']
    assert (reports[1].report['title'] ==
            'Detected modified configuration files in leapp configuration directories.')
    assert (reports[2].report['title'] ==
            'Detected modified files of the in-place upgrade tooling.')
    assert 'some/changed/file/in/framework' in reports[2].report['summary']
    assert 'some/changed/leapp/actor/file (Actor: an_actor)' in reports[2].report['summary']
